@@ -1,5 +1,5 @@
 /*
- *  WindowMaker miscelaneous function library
+ *  Window Maker miscelaneous function library
  * 
  *  Copyright (c) 1997 Alfredo K. Kojima
  * 
@@ -27,12 +27,41 @@
 #include <string.h>
 #include <errno.h>
 
-
 extern char *_WINGS_progname;
 
+
 #define MAXLINE	1024
 
 
+/*********************************************************************
+ * Returns the system error message associated with error code 'errnum'
+ *********************************************************************/
+char*
+wstrerror(int errnum)
+{
+#if defined(HAVE_STRERROR)
+    return strerror(errnum);
+#elif !defined(HAVE_STRERROR) && defined(BSD)
+    extern int errno, sys_nerr;
+#  ifndef __DECC
+    extern char *sys_errlist[];
+#  endif
+    static char buf[] = "Unknown error 12345678901234567890";
+
+    if (errno < sys_nerr)
+        return sys_errlist[errnum];
+
+    sprintf (buf, "Unknown error %d", errnum);
+    return buf;
+#else /* no strerror() and no sys_errlist[] */
+    static char buf[] = "Error 12345678901234567890";
+
+    sprintf(buf, "Error %d", errnum);
+    return buf;
+#endif
+}
+
+
 /**************************************************************************
  * Prints a fatal error message with variable arguments and terminates
  * 
@@ -53,7 +82,8 @@ wfatal(const char *msg, ...)
     fputs(_WINGS_progname, stderr);
     fputs(" fatal error: ",stderr);
     fputs(buf, stderr);
-    fflush(NULL);
+    fflush(stdout);
+    fflush(stderr);
 
     va_end(args);
 }
@@ -79,7 +109,8 @@ wwarning(const char *msg, ...)
     fputs(_WINGS_progname, stderr);
     fputs(" warning: ",stderr);
     fputs(buf, stderr);
-    fflush(NULL);
+    fflush(stdout);
+    fflush(stderr);
     
     va_end(args);
 }
@@ -96,23 +127,49 @@ wsyserror(const char *msg, ...)
 {
     va_list args;
     char buf[MAXLINE];
-#ifdef HAVE_STRERROR
     int error=errno;
-#endif
+
     va_start(args, msg);
     vsprintf(buf, msg, args);
     fflush(stdout);
     fputs(_WINGS_progname, stderr);
     fputs(" error: ", stderr);
     strcat(buf, ": ");
-#ifdef HAVE_STRERROR
-    strcat(buf, strerror(error));
+    strcat(buf, wstrerror(error));
     strcat(buf,"\n");
     fputs(buf, stderr);
-    fflush(NULL);
-#else
-    perror(buf);
-#endif
+    fflush(stderr);
+    fflush(stdout);
     va_end(args);
 }
 
+
+/*********************************************************************
+ * Prints a system error message with variable arguments, being given
+ * the error code.
+ * 
+ * error - the error code foe which to print the message
+ * msg   - message to print with optional formatting
+ * ...   - arguments to use on formatting
+ *********************************************************************/
+void 
+wsyserrorwithcode(int error, const char *msg, ...)
+{
+    va_list args;
+    char buf[MAXLINE];
+
+    va_start(args, msg);
+    vsprintf(buf, msg, args);
+    fflush(stdout);
+    fputs(_WINGS_progname, stderr);
+    fputs(" error: ", stderr);
+    strcat(buf, ": ");
+    strcat(buf, wstrerror(error));
+    strcat(buf,"\n");
+    fputs(buf, stderr);
+    fflush(stderr);
+    fflush(stdout);
+    va_end(args);
+}
+
+
