@@ -60,7 +60,7 @@ hashString(const char *key)
 
     while (*key) {
 	ret ^= *(char*)key++ << ctr;
-	ctr = (ctr + 1) % sizeof (void *);
+	ctr = (ctr + 1) % sizeof (char *);
     }
     
     return ret;
@@ -71,7 +71,7 @@ hashString(const char *key)
 static INLINE unsigned
 hashPtr(const void *key)
 {
-    return ((size_t)key / sizeof(void*));
+    return ((size_t)key / sizeof(char*));
 }
 
 
@@ -103,8 +103,8 @@ rebuildTable(WMHashTable *table)
 
     newSize = table->size*2;
 
-    table->table = wmalloc(sizeof(void*)*newSize);
-    memset(table->table, 0, sizeof(void*)*newSize);
+    table->table = wmalloc(sizeof(char*)*newSize);
+    memset(table->table, 0, sizeof(char*)*newSize);
     table->size = newSize;
     
     for (i = 0; i < oldSize; i++) {
@@ -114,7 +114,7 @@ rebuildTable(WMHashTable *table)
 	    oldArray[i] = next;
 	}
     }
-    free(oldArray);
+    wfree(oldArray);
 }
 
 
@@ -149,7 +149,7 @@ WMResetHashTable(WMHashTable *table)
 	while (item) {
 	    tmp = item->next;
 	    RELKEY(table, item);
-	    free(item);
+	    wfree(item);
 	    item = tmp;
 	}
     }
@@ -157,7 +157,7 @@ WMResetHashTable(WMHashTable *table)
     table->itemCount = 0;
     
     if (table->size > INITIAL_CAPACITY) {
-	free(table->table);
+	wfree(table->table);
 	table->size = INITIAL_CAPACITY;
 	table->table = wmalloc(sizeof(HashItem*)*table->size);
     }
@@ -175,13 +175,13 @@ WMFreeHashTable(WMHashTable *table)
 	item = table->table[i];
 	while (item) {
 	    tmp = item->next;
-	    RELKEY(table, item);
-	    free(item);
+	    RELKEY(table, item->key);
+	    wfree(item);
 	    item = tmp;
 	}
     }
-    free(table->table);
-    free(table);
+    wfree(table->table);
+    wfree(table);
 }
 
 
@@ -296,7 +296,7 @@ deleteFromList(HashTable *table, HashItem *item, const void *key)
 	
 	next = item->next;
 	RELKEY(table, item->key);
-	free(item);
+	wfree(item);
 	
 	table->itemCount--;
 
