@@ -14,6 +14,10 @@ typedef struct W_Button {
     
     WMFont *font;
 
+    WMColor *textColor;
+    WMColor *altTextColor;
+    WMColor *disTextColor;
+
     W_Pixmap *image;
     W_Pixmap *altImage;
 
@@ -117,7 +121,7 @@ WMCreateCustomButton(WMWidget *parent, int behaviourMask)
     
     bPtr->view = W_CreateView(W_VIEW(parent));
     if (!bPtr->view) {
-	free(bPtr);
+	wfree(bPtr);
 	return NULL;
     }
     bPtr->view->self = bPtr;
@@ -263,6 +267,12 @@ updateDisabledMask(WMButton *bPtr)
     }
 }
 
+void
+WMSetButtonImageDefault(WMButton *bPtr)
+{
+    WMSetButtonImage (bPtr, WMWidgetScreen(bPtr)->buttonArrow);
+    WMSetButtonAltImage (bPtr, WMWidgetScreen(bPtr)->pushedButtonArrow);
+}
 
 void
 WMSetButtonImage(WMButton *bPtr, WMPixmap *image)
@@ -334,7 +344,7 @@ void
 WMSetButtonText(WMButton *bPtr, char *text)
 {
     if (bPtr->caption)
-	free(bPtr->caption);
+	wfree(bPtr->caption);
 
     if (text!=NULL) {
 	bPtr->caption = wstrdup(text);
@@ -353,7 +363,7 @@ void
 WMSetButtonAltText(WMButton *bPtr, char *text)
 {
     if (bPtr->altCaption)
-	free(bPtr->altCaption);
+	wfree(bPtr->altCaption);
     
     if (text!=NULL) {
 	bPtr->altCaption = wstrdup(text);
@@ -367,10 +377,42 @@ WMSetButtonAltText(WMButton *bPtr, char *text)
 }
 
 
+void
+WMSetButtonTextColor(WMButton *bPtr, WMColor *color)
+{
+    if (bPtr->textColor)
+	WMReleaseColor(bPtr->textColor);
+
+    bPtr->textColor = WMRetainColor(color);
+}
+
+
+void
+WMSetButtonAltTextColor(WMButton *bPtr, WMColor *color)
+{
+    if (bPtr->altTextColor)
+	WMReleaseColor(bPtr->altTextColor);
+
+    bPtr->altTextColor = WMRetainColor(color);
+}
+
+
+void
+WMSetButtonDisabledTextColor(WMButton *bPtr, WMColor *color)
+{
+    if (bPtr->disTextColor)
+	WMReleaseColor(bPtr->disTextColor);
+
+    bPtr->disTextColor = WMRetainColor(color);
+}
+
+
 void
 WMSetButtonSelected(WMButton *bPtr, int isSelected)
 {
     bPtr->flags.selected = isSelected;
+ 
+    WMPostNotificationName(WMPushedRadioNotification, bPtr, NULL);
     
     if (bPtr->view->flags.realized) {
 	paintButton(bPtr);
@@ -539,15 +581,24 @@ static void
 paintButton(Button *bPtr)
 {
     W_Screen *scrPtr = bPtr->view->screen;
-    GC gc;
     WMReliefType relief;
     int offset;
     char *caption;
     WMPixmap *image;
-    GC textGC;
+    WMColor *textColor;
+    GC gc;
 
     gc = NULL;
     caption = bPtr->caption;
+
+    if (bPtr->flags.enabled) {
+        textColor = (bPtr->textColor!=NULL
+                     ? bPtr->textColor : scrPtr->black);
+    } else {
+        textColor = (bPtr->disTextColor!=NULL
+                     ? bPtr->disTextColor : scrPtr->darkGray);
+    }
+
     if (bPtr->flags.enabled || !bPtr->dimage)
 	image = bPtr->image;
     else
@@ -559,16 +610,19 @@ paintButton(Button *bPtr)
 	relief = WRFlat;
 
     if (bPtr->flags.selected) {
-	if (bPtr->flags.stateLight)
-	    gc = WMColorGC(scrPtr->white);
+        if (bPtr->flags.stateLight) {
+            gc = WMColorGC(scrPtr->white);
+            textColor = scrPtr->black;
+        }
 
 	if (bPtr->flags.stateChange) {
-	    if (bPtr->altCaption) {
+	    if (bPtr->altCaption)
 		caption = bPtr->altCaption;
-	    }
 	    if (bPtr->altImage)
 		image = bPtr->altImage;
-	}
+            if (bPtr->altTextColor)
+                textColor = bPtr->altTextColor;
+        }
 
 	if (bPtr->flags.statePush && bPtr->flags.bordered) {
 	    relief = WRSunken;
@@ -581,25 +635,22 @@ paintButton(Button *bPtr)
 	    relief = WRPushed;
 	    offset = 1;
 	}
-	if (bPtr->flags.pushLight)
-	    gc = WMColorGC(scrPtr->white);
+        if (bPtr->flags.pushLight) {
+            gc = WMColorGC(scrPtr->white);
+            textColor = scrPtr->black;
+        }
 
 	if (bPtr->flags.pushChange) {
-	    if (bPtr->altCaption) {
+	    if (bPtr->altCaption)
 		caption = bPtr->altCaption;
-	    }
 	    if (bPtr->altImage)
 		image = bPtr->altImage;
+            if (bPtr->altTextColor)
+                textColor = bPtr->altTextColor;
 	}
     }
-    
 
-    if (bPtr->flags.enabled)
-	textGC = WMColorGC(scrPtr->black);
-    else
-	textGC = WMColorGC(scrPtr->darkGray);
-
-    W_PaintTextAndImage(bPtr->view, True, textGC,
+    W_PaintTextAndImage(bPtr->view, True, WMColorGC(textColor),
 			(bPtr->font!=NULL ? bPtr->font : scrPtr->normalFont),
 			relief, caption, bPtr->flags.alignment, image, 
 			bPtr->flags.imagePosition, gc, offset);
@@ -746,11 +797,20 @@ destroyButton(Button *bPtr)
 	WMReleaseFont(bPtr->font);
     
     if (bPtr->caption)
-	free(bPtr->caption);
+	wfree(bPtr->caption);
 
     if (bPtr->altCaption)
-	free(bPtr->altCaption);
-    
+	wfree(bPtr->altCaption);
+
+    if (bPtr->textColor)
+        WMReleaseColor(bPtr->textColor);
+
+    if (bPtr->altTextColor)
+        WMReleaseColor(bPtr->altTextColor);
+
+    if (bPtr->disTextColor)
+        WMReleaseColor(bPtr->disTextColor);
+
     if (bPtr->image)
 	WMReleasePixmap(bPtr->image);
 
@@ -763,7 +823,7 @@ destroyButton(Button *bPtr)
     if (bPtr->altImage)
 	WMReleasePixmap(bPtr->altImage);
 
-    free(bPtr);
+    wfree(bPtr);
 }
 
 
