@@ -30,7 +30,6 @@
 
 
 
-extern _WINGsConfiguration WINGsConfiguration;
 
 
 
@@ -45,7 +44,6 @@ typedef struct TimerHandler {
 typedef struct IdleHandler {
     WMCallback		*callback;
     void 		*clientData;
-    struct IdleHandler	*next;
 } IdleHandler;
 
 
@@ -54,7 +52,6 @@ typedef struct InputHandler {
     void		*clientData;
     int			fd;
     int			mask;
-    struct InputHandler *next;
 } InputHandler;
 
 
@@ -105,9 +102,9 @@ static unsigned long eventMasks[] = {
 /* queue of timer event handlers */
 static TimerHandler *timerHandler=NULL;
 
-static IdleHandler *idleHandler=NULL;
+static WMBag *idleHandler=NULL;
 
-static InputHandler *inputHandler=NULL;
+static WMBag *inputHandler=NULL;
 
 /* hook for other toolkits or wmaker process their events */
 static WMEventHook *extraEventHandler=NULL;
@@ -116,7 +113,6 @@ static WMEventHook *extraEventHandler=NULL;
 
 #define timerPending()	(timerHandler)
 
-#define idlePending()	(idleHandler)
 
 
 static void
@@ -182,13 +178,13 @@ WMDeleteTimerWithClientData(void *cdata)
     tmp = timerHandler;
     if (tmp->clientData==cdata) {
         timerHandler = tmp->next;
-        free(tmp);
+        wfree(tmp);
     } else {
         while (tmp->next) {
             if (tmp->next->clientData==cdata) {
                 handler = tmp->next;
                 tmp->next = handler->next;
-                free(handler);
+                wfree(handler);
                 break;
             }
             tmp = tmp->next;
@@ -209,12 +205,12 @@ WMDeleteTimerHandler(WMHandlerID handlerID)
     tmp = timerHandler;
     if (tmp==handler) {
 	timerHandler = handler->next;
-	free(handler);
+	wfree(handler);
     } else {
 	while (tmp->next) {
 	    if (tmp->next==handler) {
 		tmp->next=handler->next;
-		free(handler);
+		wfree(handler);
 		break;
 	    }
 	    tmp = tmp->next;
@@ -227,25 +223,19 @@ WMDeleteTimerHandler(WMHandlerID handlerID)
 WMHandlerID
 WMAddIdleHandler(WMCallback *callback, void *cdata)
 {
-    IdleHandler *handler, *tmp;
-    
+    IdleHandler *handler;
+
     handler = malloc(sizeof(IdleHandler));
     if (!handler)
-	return NULL;
+        return NULL;
 
     handler->callback = callback;
     handler->clientData = cdata;
-    handler->next = NULL;
-    /* add callback at end of queue */
+    /* add handler at end of queue */
     if (!idleHandler) {
-	idleHandler = handler;
-    } else {
-	tmp = idleHandler;
-	while (tmp->next) {
-	    tmp = tmp->next;
-	}
-	tmp->next = handler;
+        idleHandler = WMCreateBag(16);
     }
+    WMPutInBag(idleHandler, handler);
 
     return handler;
 }
@@ -255,24 +245,16 @@ WMAddIdleHandler(WMCallback *callback, void *cdata)
 void
 WMDeleteIdleHandler(WMHandlerID handlerID)
 {
-    IdleHandler *tmp, *handler = (IdleHandler*)handlerID;
+    IdleHandler *handler = (IdleHandler*)handlerID;
+    int pos;
 
     if (!handler || !idleHandler)
-	return;
+        return;
 
-    tmp = idleHandler;
-    if (tmp == handler) {
-	idleHandler = handler->next;
-	free(handler);
-    } else {
-	while (tmp->next) {
-	    if (tmp->next == handler) {
-		tmp->next = handler->next;
-		free(handler);
-		break;
-	    }
-	    tmp = tmp->next;
-	}
+    pos = WMGetFirstInBag(idleHandler, handler);
+    if (pos != WBNotFound) {
+        wfree(handler);
+        WMDeleteFromBag(idleHandler, pos);
     }
 }
 
@@ -282,71 +264,74 @@ WMHandlerID
 WMAddInputHandler(int fd, int condition, WMInputProc *proc, void *clientData)
 {
     InputHandler *handler;
-    
+
     handler = wmalloc(sizeof(InputHandler));
-    
+
     handler->fd = fd;
     handler->mask = condition;
     handler->callback = proc;
     handler->clientData = clientData;
-    
-    handler->next = inputHandler;
-    
-    inputHandler = handler;
+
+    if (!inputHandler)
+        inputHandler = WMCreateBag(16);
+    WMPutInBag(inputHandler, handler);
 
     return handler;
 }
 
 
+
 void
 WMDeleteInputHandler(WMHandlerID handlerID)
 {
-    InputHandler *tmp, *handler = (InputHandler*)handlerID;
+    InputHandler *handler = (InputHandler*)handlerID;
+    int pos;
 
     if (!handler || !inputHandler)
 	return;
 
-    tmp = inputHandler;
-    if (tmp == handler) {
-	inputHandler = handler->next;
-	free(handler);
-    } else {
-	while (tmp->next) {
-	    if (tmp->next == handler) {
-		tmp->next = handler->next;
-		free(handler);
-		break;
-	    }
-	    tmp = tmp->next;
-	}
-    }    
+    pos = WMGetFirstInBag(inputHandler, handler);
+    if (pos != WBNotFound) {
+        wfree(handler);
+        WMDeleteFromBag(inputHandler, pos);
+    }
 }
 
 
-static void
+
+static Bool
 checkIdleHandlers()
 {
-    IdleHandler *handler, *tmp;
+    IdleHandler *handler;
+    WMBag *handlerCopy;
+    WMBagIterator iter;
 
-    if (!idleHandler) {
+    if (!idleHandler || WMGetBagItemCount(idleHandler)==0) {
 	W_FlushIdleNotificationQueue();
-	return;
+        /* make sure an observer in queue didn't added an idle handler */
+        return (idleHandler!=NULL && WMGetBagItemCount(idleHandler)>0);
     }
 
-    handler = idleHandler;
-    
-    /* we will process all idleHandlers so, empty the handler list */
-    idleHandler = NULL;
+    handlerCopy = WMCreateBag(WMGetBagItemCount(idleHandler));
+    WMAppendBag(handlerCopy, idleHandler);
+
+    for (handler = WMBagFirst(handlerCopy, &iter);
+	 iter != NULL;
+	 handler = WMBagNext(handlerCopy, &iter)) {
+        /* check if the handler still exist or was removed by a callback */
+        if (WMGetFirstInBag(idleHandler, handler) == WBNotFound)
+            continue;
 
-    while (handler) {
-	tmp = handler->next;
 	(*handler->callback)(handler->clientData);
-	/* remove the handler */
-	free(handler);
-	
-	handler = tmp;
+        WMDeleteIdleHandler(handler);
     }
+
+    WMFreeBag(handlerCopy);
+
     W_FlushIdleNotificationQueue();
+
+    /* this is not necesarrily False, because one handler can re-add itself */
+    return (WMGetBagItemCount(idleHandler)>0);
 }
 
 
@@ -357,6 +342,11 @@ checkTimerHandlers()
     TimerHandler *handler;
     struct timeval now;
 
+    if (!timerHandler) {
+        W_FlushASAPNotificationQueue();
+        return;
+    }
+
     rightNow(&now);
 
     while (timerHandler && IS_AFTER(now, timerHandler->when)) {
@@ -364,7 +354,7 @@ checkTimerHandlers()
 	timerHandler = timerHandler->next;
 	handler->next = NULL;
 	(*handler->callback)(handler->clientData);
-	free(handler);
+	wfree(handler);
     }
 
     W_FlushASAPNotificationQueue();
@@ -485,7 +475,7 @@ WMDeleteEventHandler(WMView *view, unsigned long mask, WMEventProc *eventProc,
     } else {
 	pptr->nextHandler = handler->nextHandler;
     }
-    free(handler);
+    wfree(handler);
 }
 
 
@@ -499,7 +489,7 @@ W_CleanUpEvents(WMView *view)
         
     while (ptr!=NULL) {
 	nptr = ptr->nextHandler;
-	free(ptr);
+	wfree(ptr);
 	ptr = nptr;
     }
 }
@@ -593,9 +583,12 @@ WMHandleEvent(XEvent *event)
 
     if (event->type == SelectionNotify || event->type == SelectionClear
 	|| event->type == SelectionRequest) {
-	puts("REQREWQ");
 	/* handle selection related events */
 	W_HandleSelectionEvent(event);
+	
+    } else if (event->type == ClientMessage) {
+	
+	W_HandleDNDClientMessage(toplevel, &event->xclient);
     }
 
     /* if it's a key event, redispatch it to the focused control */
@@ -650,12 +643,12 @@ WMHandleEvent(XEvent *event)
     W_RetainView(toplevel);
 
     hPtr = view->handlerList;
-			     
+
     while (hPtr!=NULL) {
 	W_EventHandler *tmp;
 
 	tmp = hPtr->nextHandler;
-	
+
 	if ((hPtr->eventMask & mask)) {
 	    (*hPtr->proc)(event, hPtr->clientData);
 	}
@@ -664,14 +657,14 @@ WMHandleEvent(XEvent *event)
     }
     
     /* pass the event to the top level window of the widget */
-    if (view->parent!=NULL) {
+    if (view->parent != NULL) {
 	vPtr = view;
-	while (vPtr->parent!=NULL)
+	while (vPtr->parent != NULL)
 	    vPtr = vPtr->parent;
 	
 	hPtr = vPtr->handlerList;
 
-	while (hPtr!=NULL) {
+	while (hPtr != NULL) {
 	
 	    if (hPtr->eventMask & mask) {
 		(*hPtr->proc)(event, hPtr->clientData);
@@ -729,29 +722,31 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
 #if defined(HAVE_POLL) && defined(HAVE_POLL_H) && !defined(HAVE_SELECT)
     struct pollfd *fds;
     InputHandler *handler;
-    int count, timeout, nfds, k, retval;
+    int count, timeout, nfds, i, retval;
 
-    for (nfds = 1, handler = inputHandler; 
-         handler != 0; handler = handler->next) nfds++;
-    
-    fds = wmalloc(nfds * sizeof(struct pollfd));
-    fds[0].fd = ConnectionNumber(dpy);
-    fds[0].events = POLLIN;
+    if (inputHandler)
+        nfds = WMGetBagItemCount(inputHandler);
+    else
+        nfds = 0;
 
-    for (k = 1, handler = inputHandler; 
-         handler; 
-         handler = handler->next, k++) {
-        fds[k].fd = handler->fd;
-        fds[k].events = 0;
-	if (handler->mask & WIReadMask)
-            fds[k].events |= POLLIN;
+    fds = wmalloc(nfds+1 * sizeof(struct pollfd));
+    /* put this to the end of array to avoid using ranges from 1 to nfds+1 */
+    fds[nfds].fd = ConnectionNumber(dpy);
+    fds[nfds].events = POLLIN;
 
-	if (handler->mask & WIWriteMask)
-            fds[k].events |= POLLOUT;
+    for (i = 0; i<nfds; i++) {
+        handler = WMGetFromBag(inputHandler, i);
+        fds[i].fd = handler->fd;
+        fds[i].events = 0;
+        if (handler->mask & WIReadMask)
+            fds[i].events |= POLLIN;
+
+        if (handler->mask & WIWriteMask)
+            fds[i].events |= POLLOUT;
 
 #if 0 /* FIXME */
-	if (handler->mask & WIExceptMask)
-	    FD_SET(handler->fd, &eset);
+        if (handler->mask & WIExceptMask)
+            FD_SET(handler->fd, &eset);
 #endif
     }
 
@@ -780,35 +775,45 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
     
     count = poll(fds, nfds, timeout);
 
-    if (count > 0) {
-	handler = inputHandler;
-        k = 1;
-	while (handler) {
+    if (count>0 && nfds>0) {
+        WMBag *handlerCopy = WMCreateBag(nfds);
+
+        for (i=0; i<nfds; i++)
+            WMPutInBag(handlerCopy, WMGetFromBag(inputHandler, i));
+
+        for (i=0; i<nfds; i++) {
 	    int mask;
 
+            handler = WMGetFromBag(handlerCopy, i);
+            /* check if the handler still exist or was removed by a callback */
+            if (WMGetFirstInBag(inputHandler, handler) == WBNotFound)
+                continue;
+
 	    mask = 0;
 
-	    if (fds[k].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI))
-		mask |= WIReadMask;
-	    
-	    if (fds[k].revents & (POLLOUT | POLLWRBAND))
-		mask |= WIWriteMask;
-	    
-	    if (fds[k].revents & (POLLHUP | POLLNVAL | POLLERR))
-		mask |= WIExceptMask;
-	    
+            if ((handler->mask & WIReadMask) &&
+                (fds[i].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI)))
+                mask |= WIReadMask;
+
+            if ((handler->mask & WIWriteMask) &&
+                (fds[i].revents & (POLLOUT | POLLWRBAND)))
+                mask |= WIWriteMask;
+
+            if ((handler->mask & WIExceptMask) &&
+                (fds[i].revents & (POLLHUP | POLLNVAL | POLLERR)))
+                mask |= WIExceptMask;
+
 	    if (mask!=0 && handler->callback) {
 		(*handler->callback)(handler->fd, mask, 
 				     handler->clientData);
 	    }
+        }
 
-	    handler = handler->next;
-            k++;
-	}
+        WMFreeBag(handlerCopy);
     }
 
-    retval = fds[0].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI);
-    free(fds);
+    retval = fds[nfds].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI);
+    wfree(fds);
 
     W_FlushASAPNotificationQueue();
 
@@ -818,9 +823,9 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
     struct timeval timeout;
     struct timeval *timeoutPtr;
     fd_set rset, wset, eset;
-    int maxfd;
+    int maxfd, nfds, i;
     int count;
-    InputHandler *handler = inputHandler;
+    InputHandler *handler;
 
     FD_ZERO(&rset);
     FD_ZERO(&wset);
@@ -829,7 +834,13 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
     FD_SET(ConnectionNumber(dpy), &rset);
     maxfd = ConnectionNumber(dpy);
 
-    while (handler) {
+    if (inputHandler)
+        nfds = WMGetBagItemCount(inputHandler);
+    else
+        nfds = 0;
+
+    for (i=0; i<nfds; i++) {
+        handler = WMGetFromBag(inputHandler, i);
 	if (handler->mask & WIReadMask)
 	    FD_SET(handler->fd, &rset);
 
@@ -841,8 +852,6 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
 
 	if (maxfd < handler->fd)
 	    maxfd = handler->fd;
-
-	handler = handler->next;
     }
 
 
@@ -868,33 +877,41 @@ W_WaitForEvent(Display *dpy, unsigned long xeventmask)
 	    return True;
 	}
     }
-    
+
     count = select(1 + maxfd, &rset, &wset, &eset, timeoutPtr);
 
-    if (count > 0) {
-	handler = inputHandler;
+    if (count>0 && nfds>0) {
+        WMBag *handlerCopy = WMCreateBag(nfds);
 
-	while (handler) {
+        for (i=0; i<nfds; i++)
+            WMPutInBag(handlerCopy, WMGetFromBag(inputHandler, i));
+
+        for (i=0; i<nfds; i++) {
 	    int mask;
 
+            handler = WMGetFromBag(handlerCopy, i);
+            /* check if the handler still exist or was removed by a callback */
+            if (WMGetFirstInBag(inputHandler, handler) == WBNotFound)
+                continue;
+
 	    mask = 0;
 
-	    if (FD_ISSET(handler->fd, &rset))
+	    if ((handler->mask & WIReadMask) && FD_ISSET(handler->fd, &rset))
 		mask |= WIReadMask;
-	    
-	    if (FD_ISSET(handler->fd, &wset))
+
+	    if ((handler->mask & WIWriteMask) && FD_ISSET(handler->fd, &wset))
 		mask |= WIWriteMask;
-	    
-	    if (FD_ISSET(handler->fd, &eset))
+
+	    if ((handler->mask & WIExceptMask) && FD_ISSET(handler->fd, &eset))
 		mask |= WIExceptMask;
-	    
+
 	    if (mask!=0 && handler->callback) {
 		(*handler->callback)(handler->fd, mask, 
 				     handler->clientData);
 	    }
+        }
 
-	    handler = handler->next;
-	}
+        WMFreeBag(handlerCopy);
     }
 
     W_FlushASAPNotificationQueue();
@@ -911,19 +928,14 @@ void
 WMNextEvent(Display *dpy, XEvent *event)
 { 
     /* Check any expired timers */
-    if (timerPending()) {
-	checkTimerHandlers();
-    }
+    checkTimerHandlers();
 
     while (XPending(dpy) == 0) {
 	/* Do idle stuff */
 	/* Do idle and timer stuff while there are no timer or X events */
-	while (!XPending(dpy) && idlePending()) {
-	    if (idlePending())
-		checkIdleHandlers();
+	while (!XPending(dpy) && checkIdleHandlers()) {
 	    /* dispatch timer events */
-	    if (timerPending())
-		checkTimerHandlers();
+            checkTimerHandlers();
 	}
 
 	/*
@@ -935,9 +947,7 @@ WMNextEvent(Display *dpy, XEvent *event)
 	W_WaitForEvent(dpy, 0);
 	
         /* Check any expired timers */
-        if (timerPending()) {
-            checkTimerHandlers();
-        }
+        checkTimerHandlers();
     }
 
     XNextEvent(dpy, event);
@@ -954,8 +964,7 @@ WMMaskEvent(Display *dpy, long mask, XEvent *event)
 
     while (!XCheckMaskEvent(dpy, mask, event)) {
 	/* Do idle stuff while there are no timer or X events */
-	while (idlePending()) {
-	    checkIdleHandlers();
+	while (checkIdleHandlers()) {
 	    if (XCheckMaskEvent(dpy, mask, event))
 		return;
 	}
@@ -981,9 +990,7 @@ WMMaskEvent(Display *dpy, long mask, XEvent *event)
 	       timeoutOrInfty);
 
         /* Check any expired timers */
-        if (timerPending()) {
-            checkTimerHandlers();
-        }
+        checkTimerHandlers();
     }
 }
 #endif
@@ -1000,8 +1007,7 @@ WMMaskEvent(Display *dpy, long mask, XEvent *event)
 { 
     while (!XCheckMaskEvent(dpy, mask, event)) {
 	/* Do idle stuff while there are no timer or X events */
-	while (idlePending()) {
-	    checkIdleHandlers();
+	while (checkIdleHandlers()) {
 	    if (XCheckMaskEvent(dpy, mask, event))
 		return;
 	}
@@ -1010,9 +1016,7 @@ WMMaskEvent(Display *dpy, long mask, XEvent *event)
 	W_WaitForEvent(dpy, mask);
 
         /* Check any expired timers */
-        if (timerPending()) {
-            checkTimerHandlers();
-        }
+        checkTimerHandlers();
     }
 }
 #endif
