@@ -427,28 +427,40 @@ loadPixmaps(WMScreen *scr)
     RDestroyImage(tmp);
 
     /* trash can */
-    tmp = RGetSubImage(image, 105, 0, 24, 24);
+    tmp = RGetSubImage(image, 104, 0, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->trashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
-    tmp = RGetSubImage(image, 105, 0, 24, 24);
+    tmp = RGetSubImage(image, 104, 0, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altTrashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
-    /* diskette */
-    tmp = RGetSubImage(image, 105, 25, 24, 24);
+    /* create dir */
+    tmp = RGetSubImage(image, 104, 24, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->createDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    tmp = RGetSubImage(image, 104, 24, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altCreateDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* diskettes */
+    tmp = RGetSubImage(image, 24, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->disketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
-    tmp = RGetSubImage(image, 105, 25, 24, 24);
+    tmp = RGetSubImage(image, 24, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
     scr->altDisketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
-
-
-    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    /* unmount */
+    tmp = RGetSubImage(image, 0, 80, 24, 24);
     RCombineImageWithColor(tmp, &white);
-    scr->altHomeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    scr->unmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    tmp = RGetSubImage(image, 0, 80, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altUnmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
 
     /* Magnifying Glass Icon for ColorPanel */
@@ -456,12 +468,6 @@ loadPixmaps(WMScreen *scr)
     RCombineImageWithColor(tmp, &gray);
     scr->magnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
-    /*
-    tmp = RGetSubImage(image, 24, 0, 40, 32);
-    RCombineImageWithColor(tmp, &white);
-    scr->altMagnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
-    RDestroyImage(tmp);
-     */
     /* ColorWheel Icon for ColorPanel */
     tmp = RGetSubImage(image, 0, 25, 24, 24);
     scr->wheelIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
@@ -540,6 +546,23 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     XGCValues gcv;
     Pixmap stipple;
     static int initialized = 0;
+    static char *atomNames[] = {
+	"_GNUSTEP_WM_ATTR",
+	    "WM_DELETE_WINDOW",
+	    "WM_PROTOCOLS",
+	    "CLIPBOARD",
+	    "XdndAware",
+	    "XdndSelection",
+	    "XdndEnter",
+	    "XdndLeave",
+	    "XdndPosition",
+	    "XdndDrop",
+	    "XdndFinished",
+	    "XdndTypeList",
+	    "XdndStatus",
+	    "WM_STATE"
+    };
+    Atom atoms[sizeof(atomNames)/sizeof(char*)];
 
     if (!initialized) {
 
@@ -623,9 +646,11 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 
     scrPtr->useMultiByte = WINGsConfiguration.useMultiByte;
 
-    scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 12);
+    scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 
+		WINGsConfiguration.defaultFontSize);
 
-    scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 12);
+    scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 
+		WINGsConfiguration.defaultFontSize);
 
     if (!scrPtr->boldFont)
 	scrPtr->boldFont = scrPtr->normalFont;
@@ -730,16 +755,38 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 	XFreePixmap(display, blank);
     }
 
-    scrPtr->internalMessage = XInternAtom(display, "_WINGS_MESSAGE", False);
+#ifdef HAVE_XINTERNATOMS
+    XInternAtoms(display, atomNames, sizeof(atomNames)/sizeof(char*), False,
+		 atoms);
+#else
+    {
+	int i;
+	for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
+	    atoms[i] = XInternAtom(display, atomNames[i], False);
+	}
+    }
+#endif
+    
+    scrPtr->attribsAtom = atoms[0];
+    
+    scrPtr->deleteWindowAtom = atoms[1];
 
-    scrPtr->attribsAtom = XInternAtom(display, "_GNUSTEP_WM_ATTR", False);
+    scrPtr->protocolsAtom = atoms[2];
     
-    scrPtr->deleteWindowAtom = XInternAtom(display, "WM_DELETE_WINDOW", False);
+    scrPtr->clipboardAtom = atoms[3];
 
-    scrPtr->protocolsAtom = XInternAtom(display, "WM_PROTOCOLS", False);
-    
-    scrPtr->clipboardAtom = XInternAtom(display, "CLIPBOARD", False);
-    
+    scrPtr->xdndAwareAtom = atoms[4];
+    scrPtr->xdndSelectionAtom = atoms[5];
+    scrPtr->xdndEnterAtom = atoms[6];
+    scrPtr->xdndLeaveAtom = atoms[7];
+    scrPtr->xdndPositionAtom = atoms[8];
+    scrPtr->xdndDropAtom = atoms[9];
+    scrPtr->xdndFinishedAtom = atoms[10];
+    scrPtr->xdndTypeListAtom = atoms[11];
+    scrPtr->xdndStatusAtom = atoms[12];
+
+    scrPtr->wmStateAtom = atoms[13];
+	
     scrPtr->rootView = W_CreateRootView(scrPtr);
 
 
@@ -941,6 +988,20 @@ WMScreenDepth(WMScreen *scr)
 }
 
 
+unsigned int 
+WMScreenWidth(WMScreen *scr)
+{
+    return scr->rootView->size.width;
+}
+
+
+unsigned int
+WMScreenHeight(WMScreen *scr)
+{
+    return scr->rootView->size.height;
+}
+
+
 void 
 WMRedisplayWidget(WMWidget *w)
 {
