@@ -31,11 +31,11 @@ typedef struct W_Scroller {
 					* dragged */
 #endif
     struct {
-	WMScrollArrowPosition arrowsPosition:3;
+	WMScrollArrowPosition arrowsPosition:4;
 
 	unsigned int horizontal:1;
 
-	WMScrollerPart hitPart:3;
+	WMScrollerPart hitPart:4;
 
 	/* */	
 	unsigned int documentFullyVisible:1;   /* document is fully visible */
@@ -67,17 +67,19 @@ typedef struct W_Scroller {
 static void destroyScroller(Scroller *sPtr);
 static void paintScroller(Scroller *sPtr);
 
-static void resizeScroller();
+static void willResizeScroller();
 static void handleEvents(XEvent *event, void *data);
 static void handleActionEvents(XEvent *event, void *data);
 
 static void handleMotion(Scroller *sPtr, int mouseX, int mouseY);
 
 
-W_ViewProcedureTable _ScrollerViewProcedures = {
+W_ViewDelegate _ScrollerViewDelegate = {
     NULL,
-	resizeScroller,
-	NULL
+	NULL,
+	NULL,
+	NULL,
+	willResizeScroller
 };
 
 
@@ -95,17 +97,19 @@ WMCreateScroller(WMWidget *parent)
 
     sPtr->view = W_CreateView(W_VIEW(parent));
     if (!sPtr->view) {
-	free(sPtr);
+	wfree(sPtr);
 	return NULL;
     }
     sPtr->view->self = sPtr;
+
+    sPtr->view->delegate = &_ScrollerViewDelegate;
     
     sPtr->flags.documentFullyVisible = 1;
     
     WMCreateEventHandler(sPtr->view, ExposureMask|StructureNotifyMask
 			 |ClientMessageMask, handleEvents, sPtr);
 
-    resizeScroller(sPtr, DEFAULT_WIDTH, DEFAULT_WIDTH);
+    W_ResizeView(sPtr->view, DEFAULT_WIDTH, DEFAULT_WIDTH);
     sPtr->flags.arrowsPosition = DEFAULT_ARROWS_POSITION;
 
     WMCreateEventHandler(sPtr->view, ButtonPressMask|ButtonReleaseMask
@@ -130,17 +134,17 @@ WMSetScrollerArrowsPosition(WMScroller *sPtr, WMScrollArrowPosition position)
 
 
 static void
-resizeScroller(WMScroller *sPtr, unsigned int width, unsigned int height)
+willResizeScroller(W_ViewDelegate *self, WMView *view, 
+		   unsigned int *width, unsigned int *height)
 {
-    if (width > height) {
+    WMScroller *sPtr = (WMScroller*)view->self;
+    
+    if (*width > *height) {
 	sPtr->flags.horizontal = 1;
-	W_ResizeView(sPtr->view, width, SCROLLER_WIDTH);
+	*height = SCROLLER_WIDTH;
     } else {
 	sPtr->flags.horizontal = 0;
-	W_ResizeView(sPtr->view, SCROLLER_WIDTH, height);
-    }
-    if (sPtr->view->flags.realized) {
-	paintScroller(sPtr);
+	*width = SCROLLER_WIDTH;
     }
 }
 
@@ -162,21 +166,21 @@ WMSetScrollerParameters(WMScroller *sPtr, float floatValue,
 {
     CHECK_CLASS(sPtr, WC_Scroller);
 
-    if (floatValue < 0)
-	sPtr->floatValue = 0;
-    else if (floatValue > 1)
-	sPtr->floatValue = 1;
+    if (floatValue < 0.0)
+	sPtr->floatValue = 0.0;
+    else if (floatValue > 1.0)
+	sPtr->floatValue = 1.0;
     else
 	sPtr->floatValue = floatValue;
 
-    if (knobProportion <= 0) {
+    if (knobProportion <= 0.0) {
 	
-	sPtr->knobProportion = 0;
+	sPtr->knobProportion = 0.0;
 	sPtr->flags.documentFullyVisible = 0;
 	
-    } else if (knobProportion >= 1) {
+    } else if (knobProportion >= 1.0) {
 	
-	sPtr->knobProportion = 1;
+	sPtr->knobProportion = 1.0;
 	sPtr->flags.documentFullyVisible = 1;
 	
     } else {
@@ -260,7 +264,7 @@ paintArrow(WMScroller *sPtr, Drawable d, int part)
 	    
 #ifndef DOUBLE_BUFFER
 	if (sPtr->flags.decrDown)
-	    gc = W_GC(scr->white);
+	    gc = WMColorGC(scr->white);
 #endif
     } else { /* increment button */
 	if (sPtr->flags.horizontal) {
@@ -300,7 +304,7 @@ paintArrow(WMScroller *sPtr, Drawable d, int part)
 		       ofs+1, 2+1, bsize+1-3, bsize-3);
 #else
 	if ((!part&&sPtr->flags.decrDown) || (part&&sPtr->flags.incrDown))
-	    XFillRectangle(scr->display, d, W_GC(scr->white),
+	    XFillRectangle(scr->display, d, WMColorGC(scr->white),
 			   ofs+1, 2+1, bsize+1-3, bsize-3);
 #endif /* DOUBLE_BUFFER */
 	W_DrawRelief(scr, d, ofs, 2, bsize, bsize, WRRaised);
@@ -324,7 +328,7 @@ paintArrow(WMScroller *sPtr, Drawable d, int part)
 		       2+1, ofs+1, bsize-3, bsize+1-3);
 #else
 	if ((!part&&sPtr->flags.decrDown) || (part&&sPtr->flags.incrDown))
-	    XFillRectangle(scr->display, d, W_GC(scr->white),
+	    XFillRectangle(scr->display, d, WMColorGC(scr->white),
 			   2+1, ofs+1, bsize-3, bsize+1-3);
 #endif /* DOUBLE_BUFFER */
 	W_DrawRelief(scr, d, 2, ofs, bsize, bsize, WRRaised);
@@ -354,9 +358,9 @@ knobLength(Scroller *sPtr)
     else
 	length = sPtr->view->size.height - 4;
 
-    if (sPtr->flags.arrowsPosition==WSAMaxEnd) {
+    if (sPtr->flags.arrowsPosition == WSAMaxEnd) {
 	length -= (SCROLLER_WIDTH - 4 + 1)*2;
-    } else if (sPtr->flags.arrowsPosition==WSAMinEnd) {
+    } else if (sPtr->flags.arrowsPosition == WSAMinEnd) {
 	length -= (SCROLLER_WIDTH - 4 + 1)*2;
     }
     
@@ -386,14 +390,14 @@ paintScroller(Scroller *sPtr)
 #ifdef DOUBLE_BUFFER
     d = XCreatePixmap(scr->display, view->window, view->size.width, 
 		      view->size.height, scr->depth);
-    XFillRectangle(scr->display, d, W_GC(scr->gray), 0, 0, 
+    XFillRectangle(scr->display, d, WMColorGC(scr->gray), 0, 0, 
 		   view->size.width, view->size.height);
 #endif
     
-    XDrawRectangle(scr->display, d, W_GC(scr->black), 0, 0,
+    XDrawRectangle(scr->display, d, WMColorGC(scr->black), 0, 0,
 		   view->size.width-1, view->size.height-1);
 #ifndef DOUBLE_BUFFER
-    XDrawRectangle(scr->display, d, W_GC(scr->gray), 1, 1,
+    XDrawRectangle(scr->display, d, WMColorGC(scr->gray), 1, 1,
 		   view->size.width-3, view->size.height-3);
 #endif
 
@@ -416,7 +420,7 @@ paintScroller(Scroller *sPtr)
 	    ofs = 0;
 	}
 	
-	knobL = knobLength(sPtr);
+	knobL = (float)knobLength(sPtr);
 
 	knobP = sPtr->floatValue * ((float)length - knobL);
 
@@ -438,7 +442,7 @@ paintScroller(Scroller *sPtr)
 	    XCopyArea(scr->display, scr->scrollerDimple->pixmap, d, 
 		      scr->copyGC, 0, 0,
 		      scr->scrollerDimple->width, scr->scrollerDimple->height,
-		      ofs+2+knobP+(int)(knobL-scr->scrollerDimple->width-1)/2,
+		      ofs+2+(int)knobP+((int)knobL-scr->scrollerDimple->width-1)/2,
 		      (view->size.height-scr->scrollerDimple->height-1)/2);
 	    
 	    /* after */
@@ -449,7 +453,7 @@ paintScroller(Scroller *sPtr)
 			       view->size.height-4);
 	} else {
 	    /* before */
-	    if (knobP>0)
+	    if (knobP>0.0)
 		XFillRectangle(scr->display, d, scr->stippleGC,
 			       2, ofs+2, view->size.width-4, (int)knobP);
 	    
@@ -463,7 +467,7 @@ paintScroller(Scroller *sPtr)
 		      scr->copyGC, 0, 0,
 		      scr->scrollerDimple->width, scr->scrollerDimple->height,
 		      (view->size.width-scr->scrollerDimple->width-1)/2,
-		      ofs+2+knobP+(int)(knobL-scr->scrollerDimple->height-1)/2);
+		      ofs+2+(int)knobP+((int)knobL-scr->scrollerDimple->height-1)/2);
 
 	    W_DrawRelief(scr, d, 2, ofs+2+(int)knobP,
 			 view->size.width-4, (int)knobL, WRRaised);
@@ -647,7 +651,7 @@ handlePush(Scroller *sPtr, int pushX, int pushY, int alternate)
 	    if (!noButtons)
 		length -= 36;
 
-	    knobP = sPtr->floatValue * (length - knobLength(sPtr));
+	    knobP = (int)(sPtr->floatValue * (float)(length-knobLength(sPtr)));
 
 	    if (sPtr->flags.arrowsPosition == WSAMinEnd)
 		sPtr->dragPoint -= 2 + (noButtons ? 0 : 36) + knobP;
@@ -655,7 +659,9 @@ handlePush(Scroller *sPtr, int pushX, int pushY, int alternate)
 		sPtr->dragPoint -= 2 + knobP;
 	}
 #endif /* STRICT_NEXT_BEHAVIOUR */
-	handleMotion(sPtr, pushX, pushY);
+        /* This does not seem necesary here since we don't know yet if the
+         * knob will be dragged later. -Dan
+        handleMotion(sPtr, pushX, pushY); */
 	break;
 
      case WSKnobSlot:
@@ -678,24 +684,25 @@ floatValueForPoint(int slotOfs, int slotLength, int knobLength, int point)
 
 #ifdef STRICT_NEXT_BEHAVIOUR
     if (point < slotOfs + knobLength/2)
-	position = slotOfs + knobLength/2;
+	position = (float)(slotOfs + knobLength/2);
     else if (point > slotOfs + slotLength - knobLength/2)
-	position = slotOfs + slotLength - knobLength/2;
+	position = (float)(slotOfs + slotLength - knobLength/2);
     else
-	position = point;
+	position = (float)point;
     
-    floatValue = (position - (slotOfs+slotLength/2)) / (float)(slotLength-knobLength);
+    floatValue = (position-(float)(slotOfs+slotLength/2))
+	/(float)(slotLength-knobLength);
 #else
     /* Adjust the last point to lie inside the knob slot */    
     if (point < slotOfs)
-	position = slotOfs;
+	position = (float)slotOfs;
     else if (point > slotOfs + slotLength)
-	position = slotOfs + slotLength;
+	position = (float)(slotOfs + slotLength);
     else
-	position = point;
+	position = (float)point;
 
     /* Compute the float value */
-    floatValue = (position - slotOfs) / (slotLength - knobLength);
+    floatValue = (position-(float)slotOfs) / (float)(slotLength-knobLength);
 #endif
     
     return floatValue;
@@ -857,6 +864,6 @@ destroyScroller(Scroller *sPtr)
 	WMDeleteTimerHandler(sPtr->timerID);
     }
 
-    free(sPtr);
+    wfree(sPtr);
 }
 
