@@ -104,7 +104,7 @@ testGradientButtons(WMScreen *scr)
     WMPixmap *pix1, *pix2;
     RImage *back;
     RColor light, dark;
-    WMColor *color;
+    WMColor *color, *altColor;
 
     windowCount++;
     
@@ -134,6 +134,9 @@ testGradientButtons(WMScreen *scr)
     pix2 = WMCreatePixmapFromRImage(scr, back, 0);
     RDestroyImage(back);
 
+    color = WMWhiteColor(scr);
+    altColor = WMCreateNamedColor(scr, "red", True);
+
     btn = WMCreateButton(win, WBTMomentaryChange);
     WMResizeWidget(btn, 60, 24);
     WMMoveWidget(btn, 20, 100);
@@ -142,8 +145,10 @@ testGradientButtons(WMScreen *scr)
     WMSetButtonImage(btn, pix1);
     WMSetButtonAltImage(btn, pix2);
     WMSetButtonText(btn, "Cool");
+    WMSetButtonTextColor(btn, color);
+    WMSetButtonAltTextColor(btn, altColor);
 
-    WMSetBalloonTextForView("This is a button", WMWidgetView(btn));
+    WMSetBalloonTextForView("This is a cool button", WMWidgetView(btn));
 
     btn = WMCreateButton(win, WBTMomentaryChange);
     WMResizeWidget(btn, 60, 24);
@@ -153,9 +158,13 @@ testGradientButtons(WMScreen *scr)
     WMSetButtonImage(btn, pix1);
     WMSetButtonAltImage(btn, pix2);
     WMSetButtonText(btn, "Button");
+    WMSetButtonTextColor(btn, color);
 
     WMSetBalloonTextForView("Este  outro balo.", WMWidgetView(btn));
 
+    WMReleaseColor(color);
+    color = WMCreateNamedColor(scr, "orange", True);
+
     btn = WMCreateButton(win, WBTMomentaryChange);
     WMResizeWidget(btn, 60, 24);
     WMMoveWidget(btn, 160, 100);
@@ -164,10 +173,14 @@ testGradientButtons(WMScreen *scr)
     WMSetButtonImage(btn, pix1);
     WMSetButtonAltImage(btn, pix2);
     WMSetButtonText(btn, "Test");
+    WMSetButtonTextColor(btn, color);
 
     WMSetBalloonTextForView("This is yet another button.\nBut the balloon has 3 lines.\nYay!",
 			    WMWidgetView(btn));
 
+    WMReleaseColor(color);
+    WMReleaseColor(altColor);
+
     WMRealizeWidget(win);
     WMMapSubwidgets(win);
     WMMapWidget(win);
@@ -330,6 +343,30 @@ testTextField(WMScreen *scr)
 }
 
 
+void
+testProgressIndicator(WMScreen *scr)
+{
+    WMWindow *win;
+    WMProgressIndicator *pPtr;
+
+    windowCount++;
+    
+    win = WMCreateWindow(scr, "testProgressIndicator");
+    WMResizeWidget(win, 292, 32);
+
+    WMSetWindowCloseAction(win, closeAction, NULL);    
+
+    pPtr = WMCreateProgressIndicator(win);
+    WMMoveWidget(pPtr, 8, 8);
+    WMSetProgressIndicatorValue(pPtr, 75);
+
+    WMRealizeWidget(win);
+    WMMapSubwidgets(win);
+    WMMapWidget(win);
+    
+}
+
+
 void
 testPullDown(WMScreen *scr)
 {
@@ -369,7 +406,6 @@ testPullDown(WMScreen *scr)
 }
 
 
-
 void
 testTabView(WMScreen *scr)
 {
@@ -431,20 +467,216 @@ testTabView(WMScreen *scr)
     WMSetTabViewItemLabel(tab, "Something");
 
     
+    frame = WMCreateFrame(win);
+    WMSetFrameRelief(frame, WRFlat);
+    label = WMCreateLabel(frame);
+    WMResizeWidget(label, 100, 100);
+    WMMoveWidget(label, 160, 40);
+    WMSetLabelText(label, "Label 4");
+    WMMapWidget(label);
+
+    tab = WMCreateTabViewItemWithIdentifier(0);
+    WMSetTabViewItemView(tab, WMWidgetView(frame));
+    WMAddItemInTabView(tabv, tab);
+    WMSetTabViewItemLabel(tab, "Bla!");
+
+
+
+    frame = WMCreateFrame(win);
+    WMSetFrameRelief(frame, WRFlat);
+    label = WMCreateLabel(frame);
+    WMResizeWidget(label, 100, 100);
+    WMMoveWidget(label, 160, 40);
+    WMSetLabelText(label, "Label fjweqklrj qwl");
+    WMMapWidget(label);
+    tab = WMCreateTabViewItemWithIdentifier(0);
+    WMSetTabViewItemView(tab, WMWidgetView(frame));    
+    WMAddItemInTabView(tabv, tab);
+    WMSetTabViewItemLabel(tab, "Weee!");
+
+    
     WMRealizeWidget(win);
     WMMapSubwidgets(win);
     WMMapWidget(win);
 }
 
 
+void
+splitViewConstrainProc(WMSplitView *sPtr, int indView,
+		       int *minSize, int *maxSize)
+{
+    switch (indView) {
+    case 0:
+    	*minSize = 20;
+	break;
+    case 1:
+    	*minSize = 40;
+    	*maxSize = 80;
+	break;
+    case 2:
+    	*maxSize = 60;
+	break;
+    default:
+    	break;
+    }
+}
+
+
+static void 
+resizeSplitView(XEvent *event, void *data)
+{
+    WMSplitView *sPtr = (WMSplitView*)data;
+
+    if (event->type == ConfigureNotify) {
+	int width = event->xconfigure.width - 10;
+        
+	if (width < WMGetSplitViewDividerThickness(sPtr))
+	    width = WMGetSplitViewDividerThickness(sPtr);
+
+	if (width != WMWidgetWidth(sPtr) ||
+            event->xconfigure.height != WMWidgetHeight(sPtr))
+	    WMResizeWidget(sPtr, width, event->xconfigure.height - 55);
+    }
+}
+
+void
+appendSubviewButtonAction(WMWidget *self, void *data)
+{
+    WMSplitView *sPtr = (WMSplitView*)data;
+    char buf[64];
+    WMLabel *label = WMCreateLabel(sPtr);
+    
+    sprintf(buf, "Subview %d", WMGetSplitViewSubviewsCount(sPtr) + 1);
+    WMSetLabelText(label, buf);
+    WMSetLabelRelief(label, WRSunken);
+    WMAddSplitViewSubview(sPtr, WMWidgetView(label));
+    WMRealizeWidget(label);
+    WMMapWidget(label);
+}
+
+void
+removeSubviewButtonAction(WMWidget *self, void *data)
+{
+    WMSplitView *sPtr = (WMSplitView*)data;
+    int count = WMGetSplitViewSubviewsCount(sPtr);
+    
+    if (count > 2) {
+    	WMView *view = WMGetSplitViewSubviewAt(sPtr, count-1);
+	WMDestroyWidget(WMWidgetOfView(view));
+    	WMRemoveSplitViewSubviewAt(sPtr, count-1);
+    }
+}
+
+void
+orientationButtonAction(WMWidget *self, void *data)
+{
+    WMSplitView *sPtr = (WMSplitView*)data;
+    WMSetSplitViewVertical(sPtr, !WMGetSplitViewVertical(sPtr));
+}
+
+void
+adjustSubviewsButtonAction(WMWidget *self, void *data)
+{
+    WMAdjustSplitViewSubviews((WMSplitView*)data);
+}
+
+void
+testSplitView(WMScreen *scr)
+{
+    WMWindow *win;
+    WMSplitView *splitv1, *splitv2;
+    WMFrame *frame;
+    WMLabel *label;
+    WMButton *button;
+
+    windowCount++;
+    
+    win = WMCreateWindow(scr, "testTabs");
+    WMResizeWidget(win, 300, 400);
+    WMSetWindowCloseAction(win, closeAction, NULL);    
+
+    frame = WMCreateFrame(win);
+    WMSetFrameRelief(frame, WRSunken);
+    WMMoveWidget(frame, 5, 5);
+    WMResizeWidget(frame, 290, 40);
+    
+    splitv1 = WMCreateSplitView(win);
+    WMMoveWidget(splitv1, 5, 50);
+    WMResizeWidget(splitv1, 290, 345);
+    WMSetSplitViewConstrainProc(splitv1, splitViewConstrainProc);
+    WMCreateEventHandler(WMWidgetView(win), StructureNotifyMask,
+                         resizeSplitView, splitv1);
+
+    button = WMCreateCommandButton(frame);
+    WMSetButtonText(button, "+");
+    WMSetButtonAction(button, appendSubviewButtonAction, splitv1);
+    WMMoveWidget(button, 10, 8);
+    WMMapWidget(button);
+    
+    button = WMCreateCommandButton(frame);
+    WMSetButtonText(button, "-");
+    WMSetButtonAction(button, removeSubviewButtonAction, splitv1);
+    WMMoveWidget(button, 80, 8);
+    WMMapWidget(button);
+    
+    button = WMCreateCommandButton(frame);
+    WMSetButtonText(button, "=");
+    WMMoveWidget(button, 150, 8);
+    WMSetButtonAction(button, adjustSubviewsButtonAction, splitv1);
+    WMMapWidget(button);
+    
+    button = WMCreateCommandButton(frame);
+    WMSetButtonText(button, "#");
+    WMMoveWidget(button, 220, 8);
+    WMSetButtonAction(button, orientationButtonAction, splitv1);
+    WMMapWidget(button);
+    
+    label = WMCreateLabel(splitv1);
+    WMSetLabelText(label, "Subview 1");
+    WMSetLabelRelief(label, WRSunken);
+    WMMapWidget(label);
+    WMAddSplitViewSubview(splitv1, WMWidgetView(label));
+
+    splitv2 = WMCreateSplitView(splitv1);
+    WMResizeWidget(splitv2, 150, 150);
+    WMSetSplitViewVertical(splitv2, True);
+
+    label = WMCreateLabel(splitv2);
+    WMSetLabelText(label, "Subview 2.1");
+    WMSetLabelRelief(label, WRSunken);
+    WMMapWidget(label);
+    WMAddSplitViewSubview(splitv2, WMWidgetView(label));
+
+    label = WMCreateLabel(splitv2);
+    WMSetLabelText(label, "Subview 2.2");
+    WMSetLabelRelief(label, WRSunken);
+    WMMapWidget(label);
+    WMAddSplitViewSubview(splitv2, WMWidgetView(label));
+
+    label = WMCreateLabel(splitv2);
+    WMSetLabelText(label, "Subview 2.3");
+    WMSetLabelRelief(label, WRSunken);
+    WMMapWidget(label);
+    WMAddSplitViewSubview(splitv2, WMWidgetView(label));
+        
+    WMMapWidget(splitv2);
+    WMAddSplitViewSubview(splitv1, WMWidgetView(splitv2));
+
+    WMRealizeWidget(win);
+    WMMapSubwidgets(win);
+    WMMapWidget(win);
+}
 
 #include "WUtil.h"
 
+
+
 int main(int argc, char **argv)
 {
     WMScreen *scr;
     WMPixmap *pixmap;
-
+    
+    
     /* Initialize the application */
     WMInitializeApplication("Test", &argc, argv);
     
@@ -486,19 +718,22 @@ int main(int argc, char **argv)
      * Put the testSomething() function you want to test here.
      */
 
-    testGradientButtons(scr);
 
-
-#if 0
     testColorWell(scr);
-
+#if 0
     testTabView(scr);
+
+    testFontPanel(scr);
+
+    testSplitView(scr);
+
+    testGradientButtons(scr);
+    testProgressIndicator(scr);
+
     testTextField(scr);
 
     testOpenFilePanel(scr);
-    testFontPanel(scr);
     testList(scr);
-    testGradientButtons(scr);
     testScrollView(scr);
 
 
