@@ -225,46 +225,49 @@ static char *blueled2_xpm[] = {
 "..####.."
 };
 
+
 /* XPM */
 static char * hand_xpm[] = {
-"22 21 17 1",
-" 	c None",
-".	c #030305",
-"+	c #101010",
-"@	c #535355",
-"#	c #7F7F7E",
-"$	c #969697",
-"%	c #B5B5B6",
-"&	c #C5C5C6",
-"*	c #D2D2D0",
-"=	c #DCDCDC",
-"-	c #E5E5E4",
-";	c #ECECEC",
-">	c #767674",
-",	c #F2F2F4",
-"'	c #676767",
-")	c #FDFDFB",
-"!	c #323234",
+"22 21 19 1",
+"       c None",
+".      c #030305",
+"+      c #7F7F7E",
+"@      c #B5B5B6",
+"#      c #C5C5C6",
+"$      c #969697",
+"%      c #FDFDFB",
+"&      c #F2F2F4",
+"*      c #E5E5E4",
+"=      c #ECECEC",
+"-      c #DCDCDC",
+";      c #D2D2D0",
+">      c #101010",
+",      c #767674",
+"'      c #676767",
+")      c #535355",
+"!      c #323234",
+"~      c #3E3C56",
+"{      c #333147",
 "                      ",
 "       .....          ",
-"     ..#%&&$.         ",
-"    .))),%..........  ",
-"   .)-)),&)))))))))$. ",
-"  .-&))))))))),,,,;;. ",
-" .=)))))))));-=***&$. ",
-" .=)))))))),..+.....  ",
-" +=)))))))))-&#.      ",
-" +=)))))))))-%>.      ",
-" +&)))))))))-%'.      ",
-" +$,,))))));...       ",
-" .#%%*;,)),*$>+       ",
-"  .'>$%&&&&$#@.       ",
-"   .!'>#$##>'.        ",
-"     ..+++++.         ",
+"     ..+@##$.         ",
+"    .%%%&@..........  ",
+"   .%*%%&#%%%%%%%%%$. ",
+"  .*#%%%%%%%%%&&&&==. ",
+" .-%%%%%%%%%=*-;;;#$. ",
+" .-%%%%%%%%&..>.....  ",
+" >-%%%%%%%%%*#+.      ",
+" >-%%%%%%%%%*@,.      ",
+" >#%%%%%%%%%*@'.      ",
+" >$&&%%%%%%=...       ",
+" .+@@;=&%%&;$,>       ",
+"  .',$@####$+).       ",
+"   .!',+$++,'.        ",
+"     ..>>>>>.         ",
 "                      ",
-"     ##@@@##          ",
-"   @@@@@@@@@@@        ",
-"     >>@@@##          ",
+"     ~~{{{~~          ",
+"   {{{{{{{{{{{        ",
+"     ~~{{{~~          ",
 "                      "};
 
 
@@ -407,20 +410,16 @@ static void
 dumpRImage(char *path, RImage *image)
 {
     FILE *f;
+    int channels = (image->format == RRGBAFormat ? 4 : 3);
 
     f = fopen(path, "w");
     if (!f) {
 	wsyserror(path);
 	return;
     }
-    fprintf(f, "%02x%02x%1x", image->width, image->height,
-	    image->data[3]!=NULL ? 4 : 3);
+    fprintf(f, "%02x%02x%1x", image->width, image->height, channels);
 
-    fwrite(image->data[0], 1, image->width * image->height, f);
-    fwrite(image->data[1], 1, image->width * image->height, f);
-    fwrite(image->data[2], 1, image->width * image->height, f);
-    if (image->data[3])
-	fwrite(image->data[3], 1, image->width * image->height, f);
+    fwrite(image->data, 1, image->width * image->height * channels, f);
 
     if (fclose(f) < 0) {
 	wsyserror(path);
@@ -434,7 +433,7 @@ isPixmap(proplist_t prop)
 {
     proplist_t p;
     char *s;
-    
+
     p = PLGetArrayElement(prop, 0);
     s = PLGetString(p);
     if (strcasecmp(&s[1], "pixmap")==0)
@@ -582,10 +581,10 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 
 	str = PLGetString(PLGetArrayElement(texture, 1));
 
-	path = wfindfileinarray(GetObjectForKey("PixmapPath"), str);
-	timage = RLoadImage(rc, path, 0);
+	if ((path=wfindfileinarray(GetObjectForKey("PixmapPath"), str))!=NULL)
+            timage = RLoadImage(rc, path, 0);
 
-	if (!timage) {
+	if (!path || !timage) {
 	    wwarning("could not load file '%s': %s", path,
 		     RMessageForError(RErrorCode));
 	} else {	
@@ -642,11 +641,10 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 
 	str = PLGetString(PLGetArrayElement(texture, 1));
 
-	path = wfindfileinarray(GetObjectForKey("PixmapPath"), str);
-	if (path)
-	    timage = RLoadImage(rc, path, 0);
+	if ((path=wfindfileinarray(GetObjectForKey("PixmapPath"), str))!=NULL)
+            timage = RLoadImage(rc, path, 0);
 
-	if (!timage) {
+	if (!path || !timage) {
 	    wwarning("could not load file '%s': %s", path ? path : str,
 		     RMessageForError(RErrorCode));
 	} else {
@@ -769,6 +767,7 @@ updatePreviewBox(_Panel *panel, int elements)
     Pixmap pix;
     GC gc;
     int colorUpdate = 0;
+    WMColor *black = WMBlackColor(scr);
 
     gc = XCreateGC(dpy, WMWidgetXID(panel->win), 0, NULL);
 
@@ -779,7 +778,7 @@ updatePreviewBox(_Panel *panel, int elements)
 	panel->preview = XCreatePixmap(dpy, WMWidgetXID(panel->win),
 				       240-4, 215-4, WMScreenDepth(scr));
 
-	color = WMGrayColor(scr);
+	color = WMCreateRGBColor(scr, 0x5100, 0x5100, 0x7100, True);
 	XFillRectangle(dpy, panel->preview, WMColorGC(color),
 		       0, 0, 240-4, 215-4);
 	WMReleaseColor(color);
@@ -790,14 +789,29 @@ updatePreviewBox(_Panel *panel, int elements)
 
     if (elements & (1<<PFOCUSED)) {
 	renderPreview(panel, gc, PFOCUSED, RBEV_RAISED2);
+	XDrawRectangle(dpy, panel->preview, WMColorGC(black),
+		       previewPositions[PFOCUSED].pos.x-1,
+		       previewPositions[PFOCUSED].pos.y-1,
+		       previewPositions[PFOCUSED].size.width,
+		       previewPositions[PFOCUSED].size.height);
 	colorUpdate |= FTITLE_COL;
     }
     if (elements & (1<<PUNFOCUSED)) {
 	renderPreview(panel, gc, PUNFOCUSED, RBEV_RAISED2);
+	XDrawRectangle(dpy, panel->preview, WMColorGC(black),
+		       previewPositions[PUNFOCUSED].pos.x-1,
+		       previewPositions[PUNFOCUSED].pos.y-1,
+		       previewPositions[PUNFOCUSED].size.width,
+		       previewPositions[PUNFOCUSED].size.height);
 	colorUpdate |= UTITLE_COL;
     }
     if (elements & (1<<POWNER)) {
 	renderPreview(panel, gc, POWNER, RBEV_RAISED2);
+	XDrawRectangle(dpy, panel->preview, WMColorGC(black),
+		       previewPositions[POWNER].pos.x-1,
+		       previewPositions[POWNER].pos.y-1,
+		       previewPositions[POWNER].size.width,
+		       previewPositions[POWNER].size.height);
 	colorUpdate |= OTITLE_COL;
     }
     if (elements & (1<<PRESIZEBAR)) {
@@ -863,6 +877,7 @@ updatePreviewBox(_Panel *panel, int elements)
     }
 
     XFreeGC(dpy, gc);
+    WMReleaseColor(black);
 }
 
 
@@ -1129,7 +1144,7 @@ changePage(WMWidget *w, void *data)
     {
 	WMColor *color;
 
-	color = WMGrayColor(scr);
+	color = WMCreateRGBColor(scr, 0x5100, 0x5100, 0x7100, True);
 	XFillRectangle(rc->dpy, panel->preview, WMColorGC(color),
 		       positions[panel->oldsection].x, 
 		       positions[panel->oldsection].y, 22, 22);
@@ -1259,6 +1274,14 @@ paintListItem(WMList *lPtr, int index, Drawable d, char *text, int state,
     WMColor *black = WMBlackColor(scr);
     TextureListItem *titem;
 
+    item = WMGetListItem(lPtr, index);
+    titem = (TextureListItem*)item->clientData;
+    if (!titem) {
+	WMReleaseColor(white);
+	WMReleaseColor(black);
+	return;
+    }
+
     width = rect->size.width;
     height = rect->size.height;
     x = rect->pos.x;
@@ -1269,13 +1292,11 @@ paintListItem(WMList *lPtr, int index, Drawable d, char *text, int state,
     else
         XClearArea(dpy, d, x, y, width, height, False);
 
-    item = WMGetListItem(lPtr, index);
-    titem = (TextureListItem*)item->clientData;
 
     if (titem->preview)
-	XCopyArea(dpy, titem->preview, d, WMColorGC(black), 0, 0, TEXPREV_WIDTH,
-		  TEXPREV_HEIGHT, x + 5, y + 5);
-
+	XCopyArea(dpy, titem->preview, d, WMColorGC(black), 0, 0, 
+		  TEXPREV_WIDTH, TEXPREV_HEIGHT, x + 5, y + 5);
+    
     if ((1 << WMGetPopUpButtonSelectedItem(panel->secP)) & titem->selectedFor)
 	WMDrawPixmap(panel->onLed, d, x + TEXPREV_WIDTH + 10, y + 6);
     else if (titem->selectedFor)
@@ -1302,7 +1323,6 @@ loadRImage(WMScreen *scr, char *path)
     FILE *f;
     RImage *image;
     int w, h, d;
-    int i;
     Pixmap pixmap;
 
     f = fopen(path, "r");
@@ -1312,9 +1332,7 @@ loadRImage(WMScreen *scr, char *path)
     fscanf(f, "%02x%02x%1x", &w, &h, &d);
 
     image = RCreateImage(w, h, d == 4);
-    for (i = 0; i < d; i++) {
-	fread(image->data[i], 1, w*h, f);
-    }
+    fread(image->data, 1, w*h*d, f);
     fclose(f);
 
     RConvertImage(WMScreenRContext(scr), image, &pixmap);
@@ -1427,7 +1445,7 @@ changeColorPage(WMWidget *w, void *data)
     if (panel->preview) {
 	WMColor *color;
 
-	color = WMGrayColor(scr);
+	color = WMCreateRGBColor(scr, 0x5100, 0x5100, 0x7100, True);
 	XFillRectangle(rc->dpy, panel->preview, WMColorGC(color),
 		       positions[panel->oldcsection].x, 
 		       positions[panel->oldcsection].y, 22, 22);
@@ -1716,7 +1734,6 @@ createPanel(Panel *p)
     panel->secP = WMCreatePopUpButton(panel->texF);
     WMResizeWidget(panel->secP, 228, 20);
     WMMoveWidget(panel->secP, 7, 7);
-    WMSetPopUpButtonSelectedItem(panel->secP, 0);
     WMAddPopUpButtonItem(panel->secP, _("Titlebar of Focused Window"));
     WMAddPopUpButtonItem(panel->secP, _("Titlebar of Unfocused Windows"));
     WMAddPopUpButtonItem(panel->secP, _("Titlebar of Focused Window's Owner"));
@@ -1726,6 +1743,7 @@ createPanel(Panel *p)
     WMAddPopUpButtonItem(panel->secP, _("Icon Background"));
 /*    WMAddPopUpButtonItem(panel->secP, _("Workspace Backgrounds"));
  */
+    WMSetPopUpButtonSelectedItem(panel->secP, 0);
     WMSetPopUpButtonAction(panel->secP, changePage, panel);
 
     panel->texLs = WMCreateList(panel->texF);
@@ -1812,7 +1830,6 @@ createPanel(Panel *p)
     panel->colP = WMCreatePopUpButton(panel->colF);
     WMResizeWidget(panel->colP, 228, 20);
     WMMoveWidget(panel->colP, 7, 7);
-    WMSetPopUpButtonSelectedItem(panel->colP, 0);
     WMAddPopUpButtonItem(panel->colP, _("Focused Window Title"));
     WMAddPopUpButtonItem(panel->colP, _("Unfocused Window Title"));
     WMAddPopUpButtonItem(panel->colP, _("Owner of Focused Window Title"));
@@ -1964,7 +1981,8 @@ setupTextureFor(WMList *list, char *key, char *defValue, char *title,
 
     titem->title = wstrdup(title);
     titem->prop = GetObjectForKey(key);
-    if (!titem->prop) {
+    if (!titem->prop || !PLIsArray(titem->prop)) {
+        /* Maybe also give a error message to stderr that the entry is bad? */
 	titem->prop = PLGetProplistWithDescription(defValue);
     } else {
 	PLRetain(titem->prop);
