@@ -46,7 +46,6 @@
 #include "dock.h"
 #include "appmenu.h"
 #include "winspector.h"
-#include "list.h"
 #include "workspace.h"
 
 #ifdef GNOME_STUFF
@@ -134,11 +133,18 @@ processEvents(int event_count)
 void
 wSetFocusTo(WScreen *scr, WWindow  *wwin)
 {
+    static WScreen *old_scr=NULL;
+
+    WWindow *old_focused;
     WWindow *focused=scr->focused_window;
     int timestamp=LastTimestamp;
     WApplication *oapp=NULL, *napp=NULL;
     int wasfocused;
 
+    if (!old_scr)
+      old_scr=scr;
+    old_focused=old_scr->focused_window;
+
     LastFocusChange = timestamp;
 
 /*
@@ -149,13 +155,13 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 */
       timestamp = CurrentTime;
 
-    if (focused)
-	oapp = wApplicationOf(focused->main_window);
+    if (old_focused)
+	oapp = wApplicationOf(old_focused->main_window);
 
     if (wwin == NULL) {
 	XSetInputFocus(dpy, scr->no_focus_win, RevertToParent, timestamp);
-	if (focused) {
-	    wWindowUnfocus(focused);
+	if (old_focused && !old_focused->flags.is_gnustep) {
+	    wWindowUnfocus(old_focused);
 	}
 	if (oapp) {
 	    wAppMenuUnmap(oapp->menu);
@@ -168,7 +174,10 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 	wKWMSendEventMessage(NULL, WKWMFocusWindow);
 #endif
 	return;
+    } else if (old_scr != scr && old_focused && !old_focused->flags.is_gnustep) {
+        wWindowUnfocus(old_focused);
     }
+
     wasfocused = wwin->flags.focused;
     napp = wApplicationOf(wwin->main_window);
 
@@ -228,7 +237,8 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 	}
     }
 
-    wWindowFocus(wwin, focused);
+    if (!wwin->flags.is_gnustep)
+	wWindowFocus(wwin, focused);
 
     if (napp && !wasfocused) {
 #ifdef USER_MENU
@@ -246,6 +256,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     wKWMSendEventMessage(wwin, WKWMFocusWindow);
 #endif
     XFlush(dpy);
+    old_scr=scr;
 }
 
 
@@ -367,7 +378,7 @@ wUnshadeWindow(WWindow  *wwin)
 		   + wwin->frame->bottom_width) {
 		XResizeWindow(dpy, wwin->frame->core->window, w, h);
 		XMoveWindow(dpy, wwin->client_win, 0, y);
-		XSync(dpy, 0);
+		XFlush(dpy);
                 if (SHADE_DELAY > 0)
                     wusleep(SHADE_DELAY*2000L/3);
 		h+=s;
@@ -459,14 +470,13 @@ wMaximizeWindow(WWindow *wwin, int directions)
 
 	new_height = (usableArea.y2-usableArea.y1)-FRAME_BORDER_WIDTH*2;
 	new_y = usableArea.y1;
-	if (WFLAGP(wwin, full_maximize))
+	if (WFLAGP(wwin, full_maximize)) {
 	    new_y -= wwin->frame->top_width;
-
+	    new_height += wwin->frame->bottom_width - 1;
+	}
     } else {
-
 	new_y = wwin->frame_y;
 	new_height = wwin->frame->core->height;
-
     }
 
     if (!WFLAGP(wwin, full_maximize)) {
@@ -1203,12 +1213,7 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
     }
 
     if (wwin->flags.inspector_open) {
-        WWindow *pwin = wwin->inspector->frame;
-
-	wWindowUnmap(pwin);
-        pwin->flags.hidden = 1;
-
-        wClientSetState(pwin, IconicState, icon->icon_win);
+	wHideInspectorForWindow(wwin);
     }
 
     wwin->flags.hidden = 1;
@@ -1263,7 +1268,7 @@ wHideOtherApplications(WWindow *awin)
 	    && wwin->frame->workspace == awin->screen_ptr->current_workspace
             && !(wwin->flags.miniaturized||wwin->flags.hidden)
             && !wwin->flags.internal_window
-            && (!wwin->flags.inspector_open || wwin->inspector->frame!=awin)
+	    && wGetWindowOfInspectorForWindow(wwin) != awin
 	    && !WFLAGP(wwin, no_hide_others)) {
 	    
 #ifdef REDUCE_APPICONS
@@ -1450,13 +1455,7 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
     wClientSetState(wwin, NormalState, None);
     wRaiseFrame(wwin->frame->core);
     if (wwin->flags.inspector_open) {
-        WWindow *pwin = wwin->inspector->frame;
-
-        pwin->flags.hidden = 0;
-        pwin->flags.mapped = 1;
-        XMapWindow(dpy, pwin->client_win);
-        XMapWindow(dpy, pwin->frame->core->window);
-        wClientSetState(pwin, NormalState, None);
+	wUnhideInspectorForWindow(wwin);
     }
 
 #ifdef GNOME_STUFF
@@ -1766,15 +1765,31 @@ void
 wSelectWindow(WWindow *wwin, Bool flag)
 {
     WScreen *scr = wwin->screen_ptr;
+    
     if (flag) {
 	wwin->flags.selected = 1;
 	XSetWindowBorder(dpy, wwin->frame->core->window, scr->white_pixel);
-	scr->selected_windows = list_cons(wwin, scr->selected_windows);
+	
+	if (WFLAGP(wwin, no_border)) {	
+	    XSetWindowBorderWidth(dpy, wwin->frame->core->window, 
+				  FRAME_BORDER_WIDTH);
+	}
+
+	if (!scr->selected_windows)
+	    scr->selected_windows = WMCreateBag(4);
+	WMPutInBag(scr->selected_windows, wwin);
     } else {
 	wwin->flags.selected = 0;
-	XSetWindowBorder(dpy, wwin->frame->core->window, 
+	XSetWindowBorder(dpy, wwin->frame->core->window,
 			 scr->frame_border_pixel);
-	scr->selected_windows = list_remove_elem(scr->selected_windows, wwin);
+
+	if (WFLAGP(wwin, no_border)) {	
+	    XSetWindowBorderWidth(dpy, wwin->frame->core->window, 0);
+	}
+
+	if (scr->selected_windows) {
+	    WMRemoveFromBag(scr->selected_windows, wwin);
+	}
     }
 }
 
