@@ -47,6 +47,9 @@
 #ifdef XDND
 #include "xdnd.h"
 #endif
+#include "wmsound.h"
+#include "WINGsP.h"
+
 
 /*
  * icon_file for the dock is got from the preferences file by
@@ -375,15 +378,15 @@ updateDockNumbers(WScreen *scr)
     length = strlen(ws_numbers);
     
     XClearArea(dpy, dicon->icon->core->window, 2, 2, 50,
-	       scr->icon_title_font->y+1, False);
+	       WMFontHeight(scr->icon_title_font)+1, False);
     
     XSetForeground(dpy, numbers_gc, scr->black_pixel);
     WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
-		 scr->icon_title_font->font, 4, 3, ws_numbers, length);
+		 scr->icon_title_font, 4, 3, ws_numbers, length);
 
     XSetForeground(dpy, numbers_gc, scr->white_pixel);
     WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
-		 scr->icon_title_font->font, 3, 2, ws_numbers, length);
+		 scr->icon_title_font, 3, 2, ws_numbers, length);
 
     XFreeGC(dpy, numbers_gc);
     free(ws_numbers);
@@ -400,7 +403,8 @@ wAppIconPaint(WAppIcon *aicon)
 
 
 # ifdef WS_INDICATOR
-    if (aicon->docked && scr->dock && aicon->yindex==0)
+    if (aicon->docked && scr->dock && scr->dock==aicon->dock &&
+        aicon->yindex==0)
 	updateDockNumbers(scr);
 # endif
     if (scr->dock_dots && aicon->docked && !aicon->running
@@ -427,30 +431,6 @@ wAppIconPaint(WAppIcon *aicon)
     }
 #endif /* HIDDENDOT */
 
-#ifdef NEWAPPICON
-    if (!wPreferences.strict_ns && aicon->icon->owner!=NULL) {
-	int active=0;
-	if (aicon->main_window==None) {
-	    active = (aicon->icon->owner->flags.focused ? 1 : 0);
-	} else {
-	    WApplication *wapp;
-
-	    wapp = wApplicationOf(aicon->main_window);
-	    if (!wapp) {
-		active = aicon->icon->owner->flags.focused;
-		wwarning("error in wAppIconPaint(). Please report it");
-	    } else {
-		active = wapp->main_window_desc->flags.focused;
-		if (wapp->main_window_desc->flags.hidden
-		    || !wapp->main_window_desc->flags.mapped)
-		  active = -1;
-	    }
-	}
-	if (active>=0)
-	  drawCorner(aicon->icon, aicon->icon->owner, active);
-    }
-#endif /* NEWAPPICON */
-
     if (aicon->omnipresent)
         drawCorner(aicon->icon);
 
@@ -480,40 +460,40 @@ wAppIconReduceAppCount(WApplication *wapp)
     /* If given a main window, check the applist
      * and remove the if it exists
      */
-	applist = wapp->app_icon->applist;
-	while (applist != NULL) {
-	        if (applist->wapp == wapp) {
-		    /* If this app owns the appicon, change the appicon's
-		     * owner to the next app in the list or NULL
-		     */
-	    if (wapp->app_icon->icon->owner == applist->wapp->main_window_desc)
-{
-			if (applist->next) {
-			    wapp->app_icon->icon->owner = applist->next->wapp->main_window_desc;
-			} else if (applist->prev) {
-			    wapp->app_icon->icon->owner = applist->prev->wapp->main_window_desc;
-			} else {
-			    wapp->app_icon->icon->owner = NULL;
-			}
-		    }
-		    if (applist->prev)
-			applist->prev->next = applist->next;
+    applist = wapp->app_icon->applist;
+    while (applist != NULL) {
+	if (applist->wapp == wapp) {
+	    /* If this app owns the appicon, change the appicon's
+	     * owner to the next app in the list or NULL
+	     */
+	    if (wapp->app_icon->icon->owner 
+		== applist->wapp->main_window_desc) {
+		if (applist->next) {
+		    wapp->app_icon->icon->owner = applist->next->wapp->main_window_desc;
+		} else if (applist->prev) {
+		    wapp->app_icon->icon->owner = applist->prev->wapp->main_window_desc;
+		} else {
+		    wapp->app_icon->icon->owner = NULL;
+		}
+	    }
+	    if (applist->prev)
+		applist->prev->next = applist->next;
 
-		    if (applist->next)
-			applist->next->prev = applist->prev;
+	    if (applist->next)
+		applist->next->prev = applist->prev;
 
 	    if (applist == wapp->app_icon->applist)
-		    	wapp->app_icon->applist = applist->next;
+		wapp->app_icon->applist = applist->next;
+	    
+	    free(applist);
 
-		    free(applist);
+	    if (wapp->app_icon->applist != NULL)
+		wapp->app_icon->main_window = wapp->app_icon->applist->wapp->main_window;
 
-		    if (wapp->app_icon->applist != NULL)
-		    	wapp->app_icon->main_window = wapp->app_icon->applist->wapp->main_window;
-
-		    return (--wapp->app_icon->num_apps);
-	    	}
-	    applist = applist->next;
+	    return (--wapp->app_icon->num_apps);
 	}
+	applist = applist->next;
+    }
     return (--wapp->app_icon->num_apps);
 }
 #endif
@@ -755,6 +735,14 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	if (!wapp)
 	    return;
 
+	if (event->xbutton.send_event &&
+	    XGrabPointer(dpy, aicon->icon->core->window, True, ButtonMotionMask
+			 |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+			 GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
+	    wwarning("pointer grab failed for appicon menu");
+	    return;
+	}
+
 	openApplicationMenu(wapp, event->xbutton.x_root,
 			    event->xbutton.y_root);
 
@@ -911,6 +899,10 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		    SlideWindow(icon->core->window, x, y, aicon->x_pos,
 				aicon->y_pos);
 		}
+
+#ifdef WMSOUND
+		wSoundPlay(WMSOUND_DOCK);
+#endif
             } else {
 		XMoveWindow(dpy, icon->core->window, x, y);
 		aicon->x_pos = x;
