@@ -1,6 +1,6 @@
 /* appicon.h- application icon
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -76,6 +76,7 @@ typedef struct WAppIcon {
     
     unsigned int gnustep_app:1;	       /* if this is a GNUstep application */
     unsigned int docked:1;
+    unsigned int omnipresent:1;        /* If omnipresent when docked in clip */
     unsigned int attracted:1;	       /* If it was attracted by the clip */
     unsigned int launching:1;
     unsigned int running:1;	       /* application is already running */
@@ -90,6 +91,9 @@ typedef struct WAppIcon {
     unsigned int destroyed:1;	       /* appicon was destroyed */
     unsigned int buggy_app:1;	       /* do not make dock rely on hints 
 					* set by app */
+    
+    unsigned int lock:1;	       /* do not allow to be destroyed */
+    
 #ifdef REDUCE_APPICONS
     unsigned int num_apps;	       /* length of applist */
 #endif
@@ -98,7 +102,7 @@ typedef struct WAppIcon {
 
 WAppIcon *wAppIconCreate(WWindow *leader_win);
 WAppIcon *wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
-				char *wm_class);
+				char *wm_class, int tile);
 
 void wAppIconDestroy(WAppIcon *aicon);
 
@@ -106,6 +110,8 @@ void wAppIconPaint(WAppIcon *aicon);
 
 Bool wAppIconChangeImage(WAppIcon *icon, char *file);
 
+void wAppIconMove(WAppIcon *aicon, int x, int y);
+
 #ifdef REDUCE_APPICONS
 unsigned int wAppIconReduceAppCount(WApplication *wapp);
 #endif
