@@ -234,14 +234,19 @@ wClientConfigure(WWindow *wwin, XConfigureRequestEvent *xcre)
 
     if (!wwin->flags.shaded) {
         /* If the window is shaded, wrong height will be set for the window */
-        if (xcre->value_mask & CWX)
-            nx = xcre->x - FRAME_BORDER_WIDTH;
+        if (xcre->value_mask & CWX) {
+            nx = xcre->x;
+	    if (!WFLAGP(wwin, no_border))
+	      nx -= FRAME_BORDER_WIDTH;
+	}
         else
             nx = wwin->frame_x;
 
-        if (xcre->value_mask & CWY)
-            ny = xcre->y - ((ofs_y < 0) ? 0 : wwin->frame->top_width)
-		- FRAME_BORDER_WIDTH;
+        if (xcre->value_mask & CWY) {
+            ny = xcre->y - ((ofs_y < 0) ? 0 : wwin->frame->top_width);
+	    if (!WFLAGP(wwin, no_border))
+	      ny -= FRAME_BORDER_WIDTH;
+	}
         else
             ny = wwin->frame_y;
 
@@ -302,7 +307,7 @@ wClientKill(WWindow *wwin)
  * Side effects:
  * 	Depends on the changed property.
  * 
- * TODO: CLIENT_LEADER, _GNUSTEP_WM_ATTR
+ * TODO: _GNUSTEP_WM_ATTR
  *---------------------------------------------------------------------- 
  */
 void
@@ -336,7 +341,7 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		free(tmp);
 	}
 	break;
-
+	
      case XA_WM_ICON_NAME:
 #ifdef KWM_HINTS
 	wKWMSendEventMessage(wwin, WKWMChangedClient);
@@ -585,7 +590,7 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 
 	    if (wwin->frame)
 		wWindowUpdateButtonImages(wwin);
-	    
+
 	} else if (event->atom==_XA_WM_COLORMAP_WINDOWS) {
 	    
 	    GetColormapWindows(wwin);
@@ -674,6 +679,13 @@ wClientGetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, Bool geometry,
     }
 
     /* some buggy apps set weird hints.. */
+    if (wwin->normal_hints->min_width <= 0) 
+      wwin->normal_hints->min_width = MIN_WINDOW_SIZE;
+    
+    if (wwin->normal_hints->min_height <= 0)
+      wwin->normal_hints->min_height = MIN_WINDOW_SIZE;
+
+
     if (wwin->normal_hints->max_width < wwin->normal_hints->min_width)
 	wwin->normal_hints->max_width = wwin->normal_hints->min_width;
 
@@ -702,12 +714,6 @@ wClientGetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, Bool geometry,
 	    wwin->normal_hints->max_aspect.y = 1;
     }
     
-    if (wwin->normal_hints->min_width <= 0) 
-      wwin->normal_hints->min_width = MIN_WINDOW_SIZE;
-    
-    if (wwin->normal_hints->min_height <= 0)
-      wwin->normal_hints->min_height = MIN_WINDOW_SIZE;
-
     if (wwin->normal_hints->min_height > wwin->normal_hints->max_height) {
 	wwin->normal_hints->min_height = wwin->normal_hints->max_height;
     }
@@ -718,6 +724,7 @@ wClientGetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, Bool geometry,
 #ifdef IGNORE_PPOSITION
     wwin->normal_hints->flags &= ~PPosition;
 #endif
+
     if (pre_icccm && !wwin->screen_ptr->flags.startup && geometry) {
 	if (wwin->normal_hints->flags & (USPosition|PPosition)) {
 	    *x = wwin->normal_hints->x;
@@ -738,7 +745,7 @@ GetColormapWindows(WWindow *wwin)
     if (wwin->cmap_windows) {
 	XFree(wwin->cmap_windows);
     }
-
+    
     wwin->cmap_windows = NULL;
     wwin->cmap_window_no = 0;
 
@@ -748,5 +755,6 @@ GetColormapWindows(WWindow *wwin)
 	wwin->cmap_window_no = 0;
 	wwin->cmap_windows = NULL;
     }
+
 #endif
 }
