@@ -66,6 +66,7 @@ typedef struct _AppSettingsPanel {
     WMButton *browseBtn;
 
     WMButton *autoLaunchBtn;
+    WMButton *lockBtn;
 
     WMButton *okBtn;
     WMButton *cancelBtn;
@@ -244,23 +245,11 @@ panelBtnCallback(WMWidget *self, void *data)
 	updateDNDCommand(panel->editedIcon, text);
 #endif
 
-#ifdef dan_remove_later
-        omnipresent = WMGetButtonSelected(panel->omnipresentBtn);
-        if (wClipMakeIconOmnipresent(panel->editedIcon, omnipresent) ==
-            WO_FAILED) {
-            wMessageDialog(panel->wwin->screen_ptr, _("Error"),
-                           _("Sorry, icon cannot be made omnipresent. "
-                             "Please make sure that no other icon is "
-                             "docked in the same position on the other "
-                             "workspaces, and the Clip is not full in "
-                             "some workspace, then try again."),
-                           _("OK"), NULL, NULL);
-            return;
-        }
-#endif
-
 	panel->editedIcon->auto_launch =
 	    WMGetButtonSelected(panel->autoLaunchBtn);
+
+	panel->editedIcon->lock =
+	    WMGetButtonSelected(panel->lockBtn);
     }
 
     if (done)
@@ -269,7 +258,7 @@ panelBtnCallback(WMWidget *self, void *data)
 
 
 #define PWIDTH	295
-#define PHEIGHT	345
+#define PHEIGHT	365
 
 
 void
@@ -304,7 +293,10 @@ ShowDockAppSettingsPanel(WAppIcon *aicon)
     font = WMBoldSystemFontOfSize(scr->wmscreen, 14);
     WMSetLabelFont(panel->nameLabel, font);
     WMReleaseFont(font);
-    WMSetLabelText(panel->nameLabel, aicon->wm_class);
+    if (aicon->wm_class && strcmp(aicon->wm_class, "DockApp")==0)
+	WMSetLabelText(panel->nameLabel, aicon->wm_instance);
+    else
+	WMSetLabelText(panel->nameLabel, aicon->wm_class);
 
     panel->autoLaunchBtn = WMCreateSwitchButton(panel->win);
     WMResizeWidget(panel->autoLaunchBtn, PWIDTH-30, 20);
@@ -313,9 +305,16 @@ ShowDockAppSettingsPanel(WAppIcon *aicon)
 		    _("Start when WindowMaker is started"));
     WMSetButtonSelected(panel->autoLaunchBtn, aicon->auto_launch);
 
+    panel->lockBtn = WMCreateSwitchButton(panel->win);
+    WMResizeWidget(panel->lockBtn, PWIDTH-30, 20);
+    WMMoveWidget(panel->lockBtn, 15, 100);
+    WMSetButtonText(panel->lockBtn,
+		    _("Lock (prevent accidental removal)"));
+    WMSetButtonSelected(panel->lockBtn, aicon->lock);
+
     panel->commandFrame = WMCreateFrame(panel->win);
     WMResizeWidget(panel->commandFrame, 275, 50);
-    WMMoveWidget(panel->commandFrame, 10, 105);
+    WMMoveWidget(panel->commandFrame, 10, 125);
     WMSetFrameTitle(panel->commandFrame, _("Application path and arguments"));
 
     panel->commandField = WMCreateTextField(panel->commandFrame);
@@ -325,7 +324,7 @@ ShowDockAppSettingsPanel(WAppIcon *aicon)
 
     panel->dndCommandFrame = WMCreateFrame(panel->win);
     WMResizeWidget(panel->dndCommandFrame, 275, 70);
-    WMMoveWidget(panel->dndCommandFrame, 10, 165);
+    WMMoveWidget(panel->dndCommandFrame, 10, 185);
     WMSetFrameTitle(panel->dndCommandFrame,
 		   _("Command for files dropped with DND"));
 
@@ -348,7 +347,7 @@ ShowDockAppSettingsPanel(WAppIcon *aicon)
 
     panel->iconFrame = WMCreateFrame(panel->win);
     WMResizeWidget(panel->iconFrame, 275, 50);
-    WMMoveWidget(panel->iconFrame, 10, 245);
+    WMMoveWidget(panel->iconFrame, 10, 265);
     WMSetFrameTitle(panel->iconFrame, _("Icon Image"));
 
     panel->iconField = WMCreateTextField(panel->iconFrame);
@@ -367,13 +366,13 @@ ShowDockAppSettingsPanel(WAppIcon *aicon)
 
     panel->okBtn = WMCreateCommandButton(panel->win);
     WMResizeWidget(panel->okBtn, 80, 26);
-    WMMoveWidget(panel->okBtn, 200, 308);
+    WMMoveWidget(panel->okBtn, 200, 328);
     WMSetButtonText(panel->okBtn, _("OK"));
     WMSetButtonAction(panel->okBtn, panelBtnCallback, panel);
 
     panel->cancelBtn = WMCreateCommandButton(panel->win);
     WMResizeWidget(panel->cancelBtn, 80, 26);
-    WMMoveWidget(panel->cancelBtn, 110, 308);
+    WMMoveWidget(panel->cancelBtn, 110, 328);
     WMSetButtonText(panel->cancelBtn, _("Cancel"));
     WMSetButtonAction(panel->cancelBtn, panelBtnCallback, panel);
 
