@@ -29,10 +29,13 @@
 #include <unistd.h>
 #include <string.h>
 
+#include <nana.h>
+
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #ifdef SHAPE
-#include <X11/extensions/shape.h>
+# include <X11/extensions/shape.h>
 #endif
 #ifdef XDND
 #include "xdnd.h"
@@ -56,7 +59,7 @@
 #include "framewin.h"
 #include "properties.h"
 #include "balloon.h"
-#include "list.h"
+
 #ifdef GNOME_STUFF
 # include "gnome.h"
 #endif
@@ -82,7 +85,9 @@ extern Atom _XA_WM_COLORMAP_NOTIFY;
 
 extern Atom _XA_WM_CHANGE_STATE;
 extern Atom _XA_WM_DELETE_WINDOW;
+extern Atom _XA_GNUSTEP_WM_ATTR;
 extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
+extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
 extern Atom _XA_WINDOWMAKER_COMMAND;
 
@@ -149,11 +154,10 @@ static int deadProcessPtr=0;
 typedef struct DeathHandler {
     WDeathHandler	*callback;
     pid_t 		pid;
-    struct DeathHandler *next;
     void 		*client_data;
 } DeathHandler;
 
-static DeathHandler *deathHandler=NULL;
+static WMBag *deathHandlers=NULL;
 
 
 
@@ -165,14 +169,15 @@ wAddDeathHandler(pid_t pid, WDeathHandler *callback, void *cdata)
     handler = malloc(sizeof(DeathHandler));
     if (!handler)
       return 0;
-    
+
     handler->pid = pid;
     handler->callback = callback;
     handler->client_data = cdata;
-    
-    handler->next = deathHandler;
-        
-    deathHandler = handler;
+
+    if (!deathHandlers)
+	deathHandlers = WMCreateBag(8);
+
+    WMPutInBag(deathHandlers, handler);
 
     return handler;
 }
@@ -182,34 +187,23 @@ wAddDeathHandler(pid_t pid, WDeathHandler *callback, void *cdata)
 void 
 wDeleteDeathHandler(WMagicNumber id)
 {
-    DeathHandler *tmp, *handler=(DeathHandler*)id;
+    DeathHandler *handler=(DeathHandler*)id;
 
-    if (!handler || !deathHandler)
+    if (!handler || !deathHandlers)
       return;
 
-    tmp = deathHandler;
-    if (tmp==handler) {
-	deathHandler = handler->next;
-	free(handler);
-    } else {
-	while (tmp->next) {
-	    if (tmp->next==handler) {
-		tmp->next=handler->next;
-		free(handler);
-		break;
-	    }
-	    tmp = tmp->next;
-	}
-    }
+    WMRemoveFromBag(deathHandlers, handler);
+
+    free(handler);
 }
 
 
 void
 DispatchEvent(XEvent *event)
 {
-    if (deathHandler)
+    if (deathHandlers)
 	handleDeadProcess(NULL);
-    
+
     if (WCHECK_STATE(WSTATE_NEED_EXIT)) {
 	WCHANGE_STATE(WSTATE_EXITING);
 	/* received SIGTERM */
@@ -232,8 +226,8 @@ DispatchEvent(XEvent *event)
      * the stuff above */
     if (!event)
 	return;
-    
-    saveTimestamp(event);    
+
+    saveTimestamp(event);
     switch (event->type) {
      case MapRequest:
 	handleMapRequest(event);
@@ -379,7 +373,7 @@ handleDeadProcess(void *foo)
 	wWindowDeleteSavedStatesForPID(deadProcesses[i].pid);
     }
 
-    if (!deathHandler) {
+    if (!deathHandlers) {
 	deadProcessPtr=0;
 	return;
     }
@@ -388,11 +382,10 @@ handleDeadProcess(void *foo)
     while (deadProcessPtr>0) {
 	deadProcessPtr--;
 
-	tmp = deathHandler;
-	while (tmp) {
-	    DeathHandler *t;
-	    
-	    t = tmp->next;
+	for (i = WMGetBagItemCount(deathHandlers)-1; i >= 0; i--) {
+	    tmp = WMGetFromBag(deathHandlers, i);
+	    if (!tmp)
+		continue;
 
 	    if (tmp->pid == deadProcesses[deadProcessPtr].pid) {
 		(*tmp->callback)(tmp->pid, 
@@ -400,7 +393,6 @@ handleDeadProcess(void *foo)
 				 tmp->client_data);
 		wDeleteDeathHandler(tmp);
 	    }
-	    tmp = t;
 	}
     }
 }
@@ -474,9 +466,8 @@ handleMapRequest(XEvent *ev)
     Window window = ev->xmaprequest.window;
 
 #ifdef DEBUG
-    printf("got map request for %x\n", (unsigned)window);
+    L("got map request for %x\n", (unsigned)window);
 #endif
-
     if ((wwin = wWindowFor(window))) {
 	if (wwin->flags.shaded) {
 	    wUnshadeWindow(wwin);
@@ -553,9 +544,8 @@ handleDestroyNotify(XEvent *event)
     Window window = event->xdestroywindow.window;
 
 #ifdef DEBUG
-    puts("got destroy notify");
-#endif	    
-
+    L("got destroy notify");
+#endif
     wwin = wWindowFor(window);
     if (wwin) {
 	wUnmanageWindow(wwin, False, True);
@@ -589,10 +579,9 @@ handleExpose(XEvent *event)
     WObjDescriptor *desc;
     XEvent ev;
 
-#ifdef DEBUG    
-    puts("got expose");
+#ifdef DEBUG
+    L("got expose");
 #endif
-
     while (XCheckTypedWindowEvent(dpy, event->xexpose.window, Expose, &ev));
 
     if (XFindContext(dpy, event->xexpose.window, wWinContext, 
@@ -605,18 +594,15 @@ handleExpose(XEvent *event)
     }
 }
 
-
 /* bindable */
 static void
 handleButtonPress(XEvent *event)
 {
     WObjDescriptor *desc;
     WScreen *scr;
-
-#ifdef DEBUG    
-    puts("got button press");
+#ifdef DEBUG
+    L("got button press");
 #endif
-
     scr = wScreenForRootWindow(event->xbutton.root);
     
 #ifdef BALLOON_TEXT
@@ -625,8 +611,7 @@ handleButtonPress(XEvent *event)
 
     
 #ifndef LITE
-    if (event->xbutton.window==scr->root_win) {
-
+    if (event->xbutton.window==scr->root_win) {	
 	if (event->xbutton.button==wPreferences.menu_button) {
 	    OpenRootMenu(scr, event->xbutton.x_root,
                          event->xbutton.y_root, False);
@@ -668,6 +653,7 @@ handleButtonPress(XEvent *event)
     }
 #endif /* !LITE */
 
+    desc = NULL;
     if (XFindContext(dpy, event->xbutton.subwindow, wWinContext,
 		     (XPointer *)&desc)==XCNOENT) {
 	if (XFindContext(dpy, event->xbutton.window, wWinContext,
@@ -676,10 +662,6 @@ handleButtonPress(XEvent *event)
 	}
     }
 
-    if (desc->handle_mousedown!=NULL) {
-	(*desc->handle_mousedown)(desc, event);
-    }
-
     if (desc->parent_type == WCLASS_WINDOW) {
 	XSync(dpy, 0);
     
@@ -704,6 +686,10 @@ handleButtonPress(XEvent *event)
 	}
     }
 
+    if (desc->handle_mousedown!=NULL) {
+	(*desc->handle_mousedown)(desc, event);
+    }
+
     /* save double-click information */
     if (scr->flags.next_click_is_not_double) {
 	scr->flags.next_click_is_not_double = 0;
@@ -719,11 +705,9 @@ static void
 handleMapNotify(XEvent *event)
 {
     WWindow *wwin;
-
 #ifdef DEBUG
-    puts("got map");
+    L("got map");
 #endif
-
     wwin = wWindowFor(event->xmap.event);
     if (wwin && wwin->client_win == event->xmap.event) {
 	if (wwin->flags.miniaturized) {
@@ -744,11 +728,9 @@ handleUnmapNotify(XEvent *event)
     WWindow *wwin;
     XEvent ev;
     Bool withdraw = False;
-
 #ifdef DEBUG
-    puts("got unmap");
+    L("got unmap");
 #endif
-
     /* only process windows with StructureNotify selected 
      * (ignore SubstructureNotify) */
     wwin = wWindowFor(event->xunmap.window);
@@ -798,9 +780,8 @@ static void
 handleConfigureRequest(XEvent *event)
 {
     WWindow *wwin;
-
 #ifdef DEBUG
-    puts("got configure request");
+    L("got configure request");
 #endif
     if (!(wwin=wWindowFor(event->xconfigurerequest.window))) {
 	/*
@@ -822,9 +803,8 @@ handlePropertyNotify(XEvent *event)
     int ji;
     unsigned int ju;
     WScreen *scr;
-
 #ifdef DEBUG
-    puts("got property notify");
+    L("got property notify");
 #endif
     if ((wwin=wWindowFor(event->xproperty.window))) {
 	if (!XGetGeometry(dpy, wwin->client_win, &jr, &ji, &ji,
@@ -852,9 +832,8 @@ handleClientMessage(XEvent *event)
 {
     WWindow *wwin;
     WObjDescriptor *desc;
-    
 #ifdef DEBUG
-    puts("got client message");
+    L("got client message");
 #endif
     /* handle transition from Normal to Iconic state */
     if (event->xclient.message_type == _XA_WM_CHANGE_STATE
@@ -912,6 +891,34 @@ handleClientMessage(XEvent *event)
 		}
 	    }
 	}
+    } else if (event->xclient.message_type == _XA_GNUSTEP_WM_ATTR) {
+	wwin = wWindowFor(event->xclient.window);
+	if (!wwin) return;
+	switch (event->xclient.data.l[0]) {
+	 case GSWindowLevelAttr:
+	   {
+	     int level = (int)event->xclient.data.l[1];
+
+	     if (WINDOW_LEVEL(wwin) != level) {
+	       ChangeStackingLevel(wwin->frame->core, level);
+	     }
+	   }
+	   break;
+	}
+    } else if (event->xclient.message_type == _XA_GNUSTEP_TITLEBAR_STATE) {
+	wwin = wWindowFor(event->xclient.window);
+	if (!wwin) return;
+	switch (event->xclient.data.l[0]) {
+	 case WMTitleBarNormal:
+	    wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
+	    break;
+	 case WMTitleBarMain:
+	    wFrameWindowChangeState(wwin->frame, WS_PFOCUSED);
+	    break;
+	 case WMTitleBarKey:
+	    wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
+	    break;
+	}
 #ifdef GNOME_STUFF
     } else if (wGNOMEProcessClientMessage(&event->xclient)) {
 	/* do nothing */
@@ -987,10 +994,31 @@ handleEnterNotify(XEvent *event)
     WObjDescriptor *desc = NULL;
     XEvent ev;
     WScreen *scr = wScreenForRootWindow(event->xcrossing.root);
-
-
 #ifdef DEBUG
-    puts("got enter notify");
+    L("got enter notify");
+#endif
+
+#ifdef VIRTUAL_DESKTOP
+    if (wPreferences.vedge_thickness) {
+        int x,y;
+        if (event->xcrossing.window == scr->virtual_edge_r) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, scr->scr_width - wPreferences.vedge_thickness - 1, event->xcrossing.y_root);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x+30, y);
+        } else if (event->xcrossing.window == scr->virtual_edge_l) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, wPreferences.vedge_thickness + 1, event->xcrossing.y_root);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x-30, y);
+        } else if (event->xcrossing.window == scr->virtual_edge_u) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, wPreferences.vedge_thickness + 1);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y-30);
+        } else if (event->xcrossing.window == scr->virtual_edge_d) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, scr->scr_height - wPreferences.vedge_thickness - 1);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y+30);
+        }
+    }
 #endif
 
     if (XCheckTypedWindowEvent(dpy, event->xcrossing.window, LeaveNotify,
@@ -1093,7 +1121,7 @@ static void
 handleLeaveNotify(XEvent *event)
 {
     WObjDescriptor *desc = NULL;
-    
+
     if (XFindContext(dpy, event->xcrossing.window, wWinContext, 
 		     (XPointer *)&desc)!=XCNOENT) {
         if(desc->handle_leavenotify)
@@ -1118,11 +1146,9 @@ handleShapeNotify(XEvent *event)
     XShapeEvent *shev = (XShapeEvent*)event;
     WWindow *wwin;
     XEvent ev;
-
 #ifdef DEBUG
-    puts("got shape notify");
+    L("got shape notify");
 #endif
-
     while (XCheckTypedWindowEvent(dpy, shev->window, event->type, &ev)) {
 	XShapeEvent *sev = (XShapeEvent*)&ev;
 
@@ -1274,66 +1300,8 @@ windowUnderPointer(WScreen *scr)
     return NULL;
 }
 
-#ifdef WEENDOZE_CYCLE
-
-static WWindow*
-nextToFocusAfter(WWindow *wwin)
-{
-    WWindow *tmp = wwin->prev;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    tmp = wwin;
-    /* start over from the beginning of the list */
-    while (tmp->next)
-	tmp = tmp->next;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    return wwin;
-}
 
 
-static WWindow*
-nextToFocusBefore(WWindow *wwin)
-{
-    WWindow *tmp = wwin->next;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    /* start over from the beginning of the list */
-    tmp = wwin;
-    while (tmp->prev)
-	tmp = tmp->prev;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    return wwin;
-}
 
 static void
 doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
@@ -1349,7 +1317,6 @@ doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
     if (!wwin)
 	return;
 
-/*    puts("IN");*/
     keymap = XGetModifierMapping(dpy);
 
     
@@ -1357,66 +1324,90 @@ doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
 		  CurrentTime);
 
     if (next) {
-	newFocused = nextToFocusAfter(wwin);
+	newFocused = NextToFocusAfter(wwin);
     } else {
-	newFocused = nextToFocusBefore(wwin); 
+	newFocused = NextToFocusBefore(wwin); 
     }
 
     scr->flags.doing_alt_tab = 1;
 
+
+    if (wPreferences.circ_raise)
+	XRaiseWindow(dpy, newFocused->frame->core->window);
     wWindowFocus(newFocused, scr->focused_window);
     oldFocused = newFocused;
-    if (wPreferences.circ_raise)
-    	wRaiseFrame(newFocused->frame->core);
 
+#if 0
     if (wPreferences.popup_switchmenu && 
 	(!scr->switch_menu || !scr->switch_menu->flags.mapped)) {
 
 	OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);
 	openedSwitchMenu = True;
     }
-	
+#endif	
     while (!done) {
 	XEvent ev;
 
 	WMMaskEvent(dpy,KeyPressMask|KeyReleaseMask|ExposureMask, &ev);
-/*	WMNextEvent(dpy, &ev);*/
+
 	if (ev.type != KeyRelease && ev.type != KeyPress) {
 	    WMHandleEvent(&ev);
 	    continue;
 	}
-/*puts("EV");*/
 	/* ignore CapsLock */
 	modifiers = ev.xkey.state & ValidModMask;
 
-	if (ev.type == KeyPress
-	    && wKeyBindings[WKBD_FOCUSNEXT].keycode == ev.xkey.keycode
-	    && wKeyBindings[WKBD_FOCUSNEXT].modifier == modifiers) {
+	if (ev.type == KeyPress) {
+	    if (wKeyBindings[WKBD_FOCUSNEXT].keycode == ev.xkey.keycode
+		&& wKeyBindings[WKBD_FOCUSNEXT].modifier == modifiers) {
+
+		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
+		newFocused = NextToFocusAfter(newFocused);
+		wWindowFocus(newFocused, oldFocused);
+		oldFocused = newFocused;
+	    
+		if (wPreferences.circ_raise) {
+		    /* restore order */
+		    CommitStacking(scr);
+		    XRaiseWindow(dpy, newFocused->frame->core->window);
+		}
+
+		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
+
+	    } else if (wKeyBindings[WKBD_FOCUSPREV].keycode == ev.xkey.keycode
+		       && wKeyBindings[WKBD_FOCUSPREV].modifier == modifiers) {
+
+		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
+		newFocused = NextToFocusBefore(newFocused);
+		wWindowFocus(newFocused, oldFocused);
+		oldFocused = newFocused;
+
+		if (wPreferences.circ_raise) {
+		    /* restore order */
+		    CommitStacking(scr);
+		    XRaiseWindow(dpy, newFocused->frame->core->window);
+		}
+		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
+
+	    } else {
+		handleKeyPress(&ev);
+	    }
+	    
+	    /*
+	    else if (wKeyBindings[WKBD_LOWER].keycode == ev.xkey.keycode
+		       && wKeyBindings[WKBD_LOWER].modifier == modifiers) {
+
+		wLowerFrame(newFocused->frame->core);
+
+	    } else if (wKeyBindings[WKBD_RAISE].keycode == ev.xkey.keycode
+		       && wKeyBindings[WKBD_RAISE].modifier == modifiers) {
 
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	    newFocused = nextToFocusAfter(newFocused);
-	    wWindowFocus(newFocused, oldFocused);
-	    oldFocused = newFocused;
-	    if (wPreferences.circ_raise)
 		wRaiseFrame(newFocused->frame->core);
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-
-	} else if (ev.type == KeyPress
-		   && wKeyBindings[WKBD_FOCUSPREV].keycode == ev.xkey.keycode
-		   && wKeyBindings[WKBD_FOCUSPREV].modifier == modifiers) {
-
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	    newFocused = nextToFocusBefore(newFocused);
-	    wWindowFocus(newFocused, oldFocused);
-	    oldFocused = newFocused;
-	    if (wPreferences.circ_raise)
-		wRaiseFrame(newFocused->frame->core);
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	}
-	if (ev.type == KeyRelease) {
+	    }*/
+	} else if (ev.type == KeyRelease) {
 	    int i;
 
-	    for (i = 0; i <= 8 * keymap->max_keypermod; i++) {
+	    for (i = 0; i < 8 * keymap->max_keypermod; i++) {
 		if (keymap->modifiermap[i] == ev.xkey.keycode &&
 		    wKeyBindings[WKBD_FOCUSNEXT].modifier 
 		    & 1<<(i/keymap->max_keypermod)) {
@@ -1426,20 +1417,22 @@ doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
 	    }
 	}
     }
-/*puts("OUT");*/
-    XFree(keymap);
+    XFreeModifiermap(keymap);
 
     XUngrabKeyboard(dpy, CurrentTime);
     wSetFocusTo(scr, newFocused);
-    scr->flags.doing_alt_tab = 0; 
+
+    if (wPreferences.circ_raise) {
+	wRaiseFrame(newFocused->frame->core);
+    	CommitStacking(scr);
+    }
+
+    scr->flags.doing_alt_tab = 0;
     if (openedSwitchMenu) 
-	OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);   
+	OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);
 }
 
 
-#endif /* WEENDOZE_CYCLE */
-
-
 
 
 static void
@@ -1449,7 +1442,7 @@ handleKeyPress(XEvent *event)
     WWindow *wwin = scr->focused_window;
     int i;
     int modifiers;
-    int command=-1;
+    int command=-1, index;
 #ifdef KEEP_XKB_LOCK_STATUS   
     XkbStateRec staterec;     
 #endif /*KEEP_XKB_LOCK_STATUS*/
@@ -1468,6 +1461,7 @@ handleKeyPress(XEvent *event)
 	    break;
 	}
     }
+        
     
     if (command < 0) {
 #ifdef LITE
@@ -1611,35 +1605,17 @@ handleKeyPress(XEvent *event)
 	}
         break;
      case WKBD_FOCUSNEXT:
-#ifdef WEENDOZE_CYCLE
-	if (wPreferences.windoze_cycling) {
+	if (wKeyBindings[WKBD_FOCUSNEXT].modifier != 0)
 	    doWindozeCycle(wwin, event, True);
-	} else
-#endif /* WEENDOZE_CYCLE */
-	{
-	    wwin = NextFocusWindow(scr);
-	    if (wwin != NULL) {
-		wSetFocusTo(scr, wwin);
-		if (wPreferences.circ_raise)
-		    wRaiseFrame(wwin->frame->core);
-	    }
-	}
+	else
+	    puts("NEXT");
 	break;
 
      case WKBD_FOCUSPREV:
-#ifdef WEENDOZE_CYCLE
-	if (wPreferences.windoze_cycling) {
+	if (wKeyBindings[WKBD_FOCUSPREV].modifier != 0)
 	    doWindozeCycle(wwin, event, False);
-	} else 
-#endif /* WEENDOZE_CYCLE */
-	{
-	    wwin = PrevFocusWindow(scr);
-	    if (wwin != NULL) {
-		wSetFocusTo(scr, wwin);
-		if (wPreferences.circ_raise)
-		    wRaiseFrame(wwin->frame->core);
-	    }
-	}
+	else
+	    puts("PREV");
 	break;
 
 #if (defined(__STDC__) && !defined(UNIXCPP)) || defined(ANSICPP)
@@ -1684,36 +1660,54 @@ handleKeyPress(XEvent *event)
      case WKBD_WINDOW9:
      case WKBD_WINDOW10:
 #endif
-        if ( scr->shortcutSelectedWindows[command-WKBD_WINDOW1]) {
-            LinkedList *list = scr->shortcutSelectedWindows[command-WKBD_WINDOW1];
+
+#define INITBAG(bag)  if (bag) WMEmptyBag(bag); else bag = WMCreateBag(4)
+
+	index = command-WKBD_WINDOW1;
+	
+        if (scr->shortcutWindows[index]) {
+            WMBag *list = scr->shortcutWindows[index];
             int cw;
+	    int count = WMGetBagItemCount(list);
+	    WWindow *twin;
+	    WMBagIterator iter;
+	    WWindow *wwin;
 
             wUnselectWindows(scr);
-            if (scr->shortcutWindow[command-WKBD_WINDOW1])
-                wMakeWindowVisible(scr->shortcutWindow[command-WKBD_WINDOW1]);
             cw = scr->current_workspace;
-            while (list) {
-                wWindowChangeWorkspace(list->head, cw);
-                wMakeWindowVisible(list->head);
-                wSelectWindow(list->head, True);
-                list = list->tail;
-            }
-        } else if (scr->shortcutWindow[command-WKBD_WINDOW1]){
 
-            wMakeWindowVisible(scr->shortcutWindow[command-WKBD_WINDOW1]);
+	    for (wwin = WMBagLast(list, &iter);
+		 iter != NULL;
+		 wwin = WMBagPrevious(list, &iter)) {
+			
+		if (count > 1)
+		    wWindowChangeWorkspace(wwin, cw);
+
+                wMakeWindowVisible(wwin);
+
+		if (count > 1)
+		    wSelectWindow(wwin, True);
+            }
+	    
+	    /* rotate the order of windows, to create a cycling effect */
+	    twin = WMBagFirst(list, &iter);
+	    WMRemoveFromBag(list, twin);
+	    WMPutInBag(list, twin);
 
         } else if (wwin && ISMAPPED(wwin) && ISFOCUSED(wwin)) {
 
-            scr->shortcutWindow[command-WKBD_WINDOW1] = wwin;
-            if (wwin->flags.selected /* && scr->selected_windows */ ) {
-                LinkedList *sl;
+	    INITBAG(scr->shortcutWindows[index]);
+            WMPutInBag(scr->shortcutWindows[index], wwin);
 
-                sl = scr->selected_windows;
-                list_free(scr->shortcutSelectedWindows[command-WKBD_WINDOW1]);
+            if (wwin->flags.selected && scr->selected_windows) {
+                WMBag *selwins = scr->selected_windows;
+		int i;
 
-                while (sl) {
-                    scr->shortcutSelectedWindows[command-WKBD_WINDOW1] = list_cons(sl->head,scr->shortcutSelectedWindows[command-WKBD_WINDOW1]);
-                    sl = sl->tail;
+		for (i = 0; i < WMGetBagItemCount(selwins); i++) {
+		    WWindow *tmp = WMGetFromBag(selwins, i);
+
+		    if (tmp != wwin)
+			WMPutInBag(scr->shortcutWindows[index], tmp);
                 }
             }
             wSelectWindow(wwin, !wwin->flags.selected);
@@ -1722,20 +1716,23 @@ handleKeyPress(XEvent *event)
             wSelectWindow(wwin, !wwin->flags.selected);
             XFlush(dpy);
 
-        } else if (scr->selected_windows) {
+        } else if (scr->selected_windows 
+		   && WMGetBagItemCount(scr->selected_windows)) {
 
-            if (wwin->flags.selected /* && scr->selected_windows */ ) {
-                LinkedList *sl;
+            if (wwin->flags.selected && scr->selected_windows) {
+                WMBag *selwins = scr->selected_windows;
+		int i;
 
-                sl = scr->selected_windows;
-                list_free(scr->shortcutSelectedWindows[command-WKBD_WINDOW1]);
+                INITBAG(scr->shortcutWindows[index]);
 
-                while (sl) {
-                    scr->shortcutSelectedWindows[command-WKBD_WINDOW1] = list_cons(sl->head,scr->shortcutSelectedWindows[command-WKBD_WINDOW1]);
-                    sl = sl->tail;
+		for (i = 0; i < WMGetBagItemCount(selwins); i++) {
+		    WWindow *tmp = WMGetFromBag(selwins, i);
+
+                    WMPutInBag(scr->shortcutWindows[index],  tmp);
                 }
             }
         }
+#undef INITBAG
 
         break;
      case WKBD_NEXTWSLAYER:
@@ -1803,11 +1800,9 @@ handleMotionNotify(XEvent *event)
             event->xmotion.x_root >= (scr->scr_width - 2) ||
             event->xmotion.y_root <= 1 ||
             event->xmotion.y_root >= (scr->scr_height - 2)) {
-
 #ifdef DEBUG
-            puts("pointer at screen edge");
+            L("pointer at screen edge");
 #endif
-
             menu = wMenuUnderPointer(scr);
             if (menu!=NULL)
                 wMenuScroll(menu, event);
