@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdarg.h>
 #include <pwd.h>
 #include <math.h>
 #include <time.h>
@@ -48,7 +49,6 @@
 #include "xutil.h"
 #include "xmodifier.h"
 
-#include "list.h"
 
 /**** global variables *****/
 
@@ -116,6 +116,7 @@ MakeCPPArgs(char *path)
     int i;
     char buffer[MAXLINE], *buf, *line;
     Visual *visual;
+    char *tmp;
     
     line = wmalloc(MAXLINE);
     *line = 0;
@@ -150,13 +151,21 @@ MakeCPPArgs(char *path)
     putidef(line, " -DSCR_HEIGHT=", 
 	    HeightOfScreen(DefaultScreenOfDisplay(dpy)));
 
-    #if 0
-    strcpy(buffer, path);    
-    buf = strrchr(buffer, '/');
-    if (buf) *buf = 0; /* trunc filename */
-    putdef(line, " -I", buffer);
-    #endif
-
+    /* put the dir where the menu is being read from to the
+     * search path */
+    if (path) {
+	tmp = wstrdup(path);
+	buf = strchr(tmp+1, ' ');
+	if (buf) {
+	    *buf = 0;
+	}
+	buf = strrchr(tmp, '/');
+	if (buf) {
+	    *buf = 0; /* trunc filename */
+	    putdef(line, " -I", tmp);
+	}
+	free(tmp);
+    }
 
 
     /* this should be done just once, but it works this way */
@@ -191,69 +200,64 @@ MakeCPPArgs(char *path)
 #endif /* USECPP */
 
 
-WWindow*
-NextFocusWindow(WScreen *scr)
-{
-    WWindow *tmp, *wwin, *closest, *min;
-    Window d;
 
-    if (!(wwin = scr->focused_window))
-        return NULL;
-    tmp = wwin->prev;
-    closest = NULL;
-    min = wwin;
-    d = 0xffffffff;
+
+
+WWindow*
+NextToFocusAfter(WWindow *wwin)
+{
+    WWindow *tmp = wwin->prev;
+
     while (tmp) {
-        if (wWindowCanReceiveFocus(tmp) 
-	    && (!WFLAGP(tmp, skip_window_list)|| tmp->flags.internal_window)) {
-	    if (min->client_win > tmp->client_win)
-	      min = tmp;
-	    if (tmp->client_win > wwin->client_win
-		&& (!closest
-		    || (tmp->client_win - wwin->client_win) < d)) {
-		closest = tmp;
-		d = tmp->client_win - wwin->client_win;
-	    }
+	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
+	    return tmp;
 	}
 	tmp = tmp->prev;
     }
-    if (!closest||closest==wwin)
-      return min;
-    return closest;
+
+    tmp = wwin;
+    /* start over from the beginning of the list */
+    while (tmp->next)
+	tmp = tmp->next;
+
+    while (tmp && tmp != wwin) {
+	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
+	    return tmp;
+	}
+	tmp = tmp->prev;
+    }
+
+    return wwin;
 }
 
 
 WWindow*
-PrevFocusWindow(WScreen *scr)
+NextToFocusBefore(WWindow *wwin)
 {
-    WWindow *tmp, *wwin, *closest, *max;
-    Window d;
-    
-    if (!(wwin = scr->focused_window))
-        return NULL;
-    tmp = wwin->prev;
-    closest = NULL;
-    max = wwin;
-    d = 0xffffffff;
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) &&
-            (!WFLAGP(tmp, skip_window_list) || tmp->flags.internal_window)) {
-	    if (max->client_win < tmp->client_win)
-	      max = tmp;
-	    if (tmp->client_win < wwin->client_win
-		&& (!closest
-		    || (wwin->client_win - tmp->client_win) < d)) {
-		closest = tmp;
-		d = wwin->client_win - tmp->client_win;
-	    }
-	}
-	tmp = tmp->prev;
-    }
-    if (!closest||closest==wwin)
-      return max;
-    return closest;
-}
+    WWindow *tmp = wwin->next;
 
+    while (tmp) {
+	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
+	    return tmp;
+	}
+	tmp = tmp->next;
+    }
+
+    /* start over from the beginning of the list */
+    tmp = wwin;
+    while (tmp->prev)
+	tmp = tmp->prev;
+
+    while (tmp && tmp != wwin) {
+	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
+
+	    return tmp;
+	}
+	tmp = tmp->next;
+    }
+
+    return wwin;
+}
 
 
 #if 0
@@ -682,26 +686,26 @@ next_token(char *word, char **next)
 void
 ParseCommand(char *command, char ***argv, int *argc)
 {
-    LinkedList *list = NULL;
+    WMBag *bag = WMCreateBag(4);
     char *token, *line;
-    int count, i;
+    int count, j;
 
     line = command;
     do {
 	token = next_token(line, &line);
-	if (token) {	    
-	    list = list_cons(token, list);
+	if (token) {
+	    WMPutInBag(bag, token);
 	}
     } while (token!=NULL && line!=NULL);
 
-    count = list_length(list);
+    count = WMGetBagItemCount(bag);
     *argv = wmalloc(sizeof(char*)*count);
-    i = count;
-    while (list!=NULL) {
-	(*argv)[--i] = list->head;
-	list_remove_head(&list);
+    for (j = 0; j < count; j++) {
+	(*argv)[j] = WMGetFromBag(bag, j);
     }
     *argc = count;
+
+    WMFreeBag(bag);
 }
 
 #if 0
@@ -1160,6 +1164,19 @@ keysymToString(KeySym keysym, unsigned int state)
 }
 #endif
 
+static char *
+appendrealloc(char *a, char *b)
+{
+    if (a == NULL)
+	return wstrdup(b);
+    else {
+	char *c = wstrappend(a, b);
+	free(a);
+	return c;
+    }
+}
+
+
 char*
 GetShortcutString(char *text)
 {
@@ -1185,33 +1202,33 @@ GetShortcutString(char *text)
 	modmask |= mod;
 	
 	if (strcasecmp(text, "Meta")==0) {
-	    buffer = wstrappend(buffer, "M+");
+	    buffer = appendrealloc(buffer, "M+");
 	} else if (strcasecmp(text, "Alt")==0) {
-	    buffer = wstrappend(buffer, "A+");
+	    buffer = appendrealloc(buffer, "A+");
 	} else if (strcasecmp(text, "Shift")==0) {
-	    buffer = wstrappend(buffer, "Sh+");
+	    buffer = appendrealloc(buffer, "Sh+");
 	} else if (strcasecmp(text, "Mod1")==0) {
-	    buffer = wstrappend(buffer, "M1+"); 
+	    buffer = appendrealloc(buffer, "M1+"); 
 	} else if (strcasecmp(text, "Mod2")==0) {
-	    buffer = wstrappend(buffer, "M2+"); 
+	    buffer = appendrealloc(buffer, "M2+"); 
 	} else if (strcasecmp(text, "Mod3")==0) {
-	    buffer = wstrappend(buffer, "M3+");
+	    buffer = appendrealloc(buffer, "M3+");
 	} else if (strcasecmp(text, "Mod4")==0) {
-	    buffer = wstrappend(buffer, "M4+"); 
+	    buffer = appendrealloc(buffer, "M4+"); 
 	} else if (strcasecmp(text, "Mod5")==0) {
-	    buffer = wstrappend(buffer, "M5+"); 
+	    buffer = appendrealloc(buffer, "M5+"); 
 	} else if (strcasecmp(text, "Control")==0) {
 	    control = 1;
 	} else {
-	    buffer = wstrappend(buffer, text);
+	    buffer = appendrealloc(buffer, text);
 	}
 	text = k+1;
     }
 
     if (control) {
-	buffer = wstrappend(buffer, "^");
+	buffer = appendrealloc(buffer, "^");
     }
-    buffer = wstrappend(buffer, text);
+    buffer = appendrealloc(buffer, text);
     
     /* get key */
 /*    ksym = XStringToKeysym(text);
@@ -1385,38 +1402,3 @@ SendHelperMessage(WScreen *scr, char type, int workspace, char *msg)
     }
     free(buffer);
 }
-
-
-void
-ExecuteShellCommand(WScreen *scr, char *command)
-{
-    static char *shell = NULL;
-    pid_t pid;
-
-    /*
-     * This have a problem: if the shell is tcsh (not sure about others)
-     * and ~/.tcshrc have /bin/stty erase ^H somewhere on it, the shell
-     * will block and the command will not be executed.
-    if (!shell) {
-	shell = getenv("SHELL");
-	if (!shell)
-	    shell = "/bin/sh";
-    }
-     */
-    shell = "/bin/sh";
-
-    pid = fork();
-    if (pid==0) {
-
-	SetupEnvironment(scr);
-
-#ifdef HAVE_SETPGID
-	setpgid(0, 0);
-#endif
-	execl(shell, shell, "-c", command, NULL);
-	wsyserror("could not execute %s -c %s", shell, command);
-	Exit(-1);
-    } else if (pid < 0) {
-	wsyserror("cannot fork a new process");
-    }
-}
