@@ -126,8 +126,11 @@ extern Atom _XA_WINDOWMAKER_STATE;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
 extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
 extern Atom _XA_WINDOWMAKER_COMMAND;
+extern Atom _XA_WINDOWMAKER_ICON_SIZE;
+extern Atom _XA_WINDOWMAKER_ICON_TILE;
 
 extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
+extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
 
 #ifdef OFFIX_DND
 extern Atom _XA_DND_PROTOCOL;
@@ -626,6 +629,29 @@ wScreenForWindow(Window window)
 }
 
 
+static char *atomNames[] = {
+    "WM_STATE", 
+    "WM_CHANGE_STATE", 
+    "WM_PROTOCOLS", 
+    "WM_TAKE_FOCUS", 
+    "WM_DELETE_WINDOW", 
+    "WM_SAVE_YOURSELF", 
+    "WM_CLIENT_LEADER", 
+    "WM_COLORMAP_WINDOWS", 
+    "WM_COLORMAP_NOTIFY", 
+    GNUSTEP_WM_ATTR_NAME, 
+    "_WINDOWMAKER_MENU", 
+    "_WINDOWMAKER_STATE", 
+    "_WINDOWMAKER_WM_PROTOCOLS", 
+    GNUSTEP_WM_MINIATURIZE_WINDOW,
+    "_WINDOWMAKER_WM_FUNCTION",
+    "_WINDOWMAKER_NOTICEBOARD",
+    "_WINDOWMAKER_COMMAND",
+    "_WINDOWMAKER_ICON_SIZE",
+    "_WINDOWMAKER_ICON_TILE",
+    GNUSTEP_TITLEBAR_STATE
+};
+
 
 /*
  *----------------------------------------------------------
@@ -643,6 +669,7 @@ StartUp(Bool defaultScreenOnly)
     WWorkspaceState *ws_state;
     struct sigaction sig_action;
     int j, max;
+    Atom atom[sizeof(atomNames)/sizeof(char*)];
 
     /*
      * Ignore CapsLock in modifiers
@@ -664,34 +691,48 @@ StartUp(Bool defaultScreenOnly)
 
 /*    _XA_VERSION = XInternAtom(dpy, "VERSION", False);*/
 
-    _XA_WM_STATE = XInternAtom(dpy, "WM_STATE", False);
-    _XA_WM_CHANGE_STATE = XInternAtom(dpy, "WM_CHANGE_STATE", False);
-    _XA_WM_PROTOCOLS = XInternAtom(dpy, "WM_PROTOCOLS", False);
-    _XA_WM_TAKE_FOCUS = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
-    _XA_WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
-    _XA_WM_SAVE_YOURSELF = XInternAtom(dpy, "WM_SAVE_YOURSELF", False);    
-    _XA_WM_CLIENT_LEADER = XInternAtom(dpy, "WM_CLIENT_LEADER", False);
-    _XA_WM_COLORMAP_WINDOWS = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
-    _XA_WM_COLORMAP_NOTIFY = XInternAtom(dpy, "WM_COLORMAP_NOTIFY", False);
+#ifdef HAVE_XINTERNATOMS
+    XInternAtoms(dpy, atomNames, sizeof(atomNames)/sizeof(char*),
+		 False, atom);
+#else
 
-    _XA_GNUSTEP_WM_ATTR = XInternAtom(dpy, GNUSTEP_WM_ATTR_NAME, False);
-
-    _XA_WINDOWMAKER_MENU = XInternAtom(dpy, "_WINDOWMAKER_MENU", False);
-    _XA_WINDOWMAKER_STATE = XInternAtom(dpy, "_WINDOWMAKER_STATE", False);
-
-    _XA_WINDOWMAKER_WM_PROTOCOLS =
-	XInternAtom(dpy, "_WINDOWMAKER_WM_PROTOCOLS", False);
-
-    _XA_GNUSTEP_WM_MINIATURIZE_WINDOW = 
-	XInternAtom(dpy, GNUSTEP_WM_MINIATURIZE_WINDOW, False);
+    {
+	int i;
+	for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
+	    atom[i] = XInternAtom(dpy, atomNames[i], False);
+	}
+    }
+#endif
     
-    _XA_WINDOWMAKER_WM_FUNCTION = XInternAtom(dpy, "_WINDOWMAKER_WM_FUNCTION",
-					      False);
+    _XA_WM_STATE = atom[0];
+    _XA_WM_CHANGE_STATE = atom[1];
+    _XA_WM_PROTOCOLS = atom[2];
+    _XA_WM_TAKE_FOCUS = atom[3];
+    _XA_WM_DELETE_WINDOW = atom[4];
+    _XA_WM_SAVE_YOURSELF = atom[5];
+    _XA_WM_CLIENT_LEADER = atom[6];
+    _XA_WM_COLORMAP_WINDOWS = atom[7];
+    _XA_WM_COLORMAP_NOTIFY = atom[8];
 
-    _XA_WINDOWMAKER_NOTICEBOARD = XInternAtom(dpy, "_WINDOWMAKER_NOTICEBOARD",
-					      False);
+    _XA_GNUSTEP_WM_ATTR = atom[9];
+
+    _XA_WINDOWMAKER_MENU = atom[10];
+    _XA_WINDOWMAKER_STATE = atom[11];
+
+    _XA_WINDOWMAKER_WM_PROTOCOLS = atom[12];
+
+    _XA_GNUSTEP_WM_MINIATURIZE_WINDOW = atom[13];
     
-    _XA_WINDOWMAKER_COMMAND = XInternAtom(dpy, "_WINDOWMAKER_COMMAND", False);
+    _XA_WINDOWMAKER_WM_FUNCTION = atom[14];
+
+    _XA_WINDOWMAKER_NOTICEBOARD = atom[15];
+    
+    _XA_WINDOWMAKER_COMMAND = atom[16];
+
+    _XA_WINDOWMAKER_ICON_SIZE = atom[17];
+    _XA_WINDOWMAKER_ICON_TILE = atom[18];
+
+    _XA_GNUSTEP_TITLEBAR_STATE = atom[19];
 
 #ifdef OFFIX_DND
     _XA_DND_SELECTION = XInternAtom(dpy, "DndSelection", False);
@@ -703,13 +744,25 @@ StartUp(Bool defaultScreenOnly)
 
 
     /* cursors */
+#ifdef DEFINABLE_CURSOR
+    wCursor[WCUR_NORMAL] = None;
+#else
     wCursor[WCUR_NORMAL] = XCreateFontCursor(dpy, XC_left_ptr);    
+#endif
+    wCursor[WCUR_ROOT] = XCreateFontCursor(dpy, XC_left_ptr);
     wCursor[WCUR_ARROW] = XCreateFontCursor(dpy, XC_top_left_arrow);
     wCursor[WCUR_MOVE] = XCreateFontCursor(dpy, XC_fleur);
     wCursor[WCUR_RESIZE] = XCreateFontCursor(dpy, XC_sizing);
+    wCursor[WCUR_TOPLEFTRESIZE] = XCreateFontCursor(dpy, XC_top_left_corner);
+    wCursor[WCUR_TOPRIGHTRESIZE] = XCreateFontCursor(dpy, XC_top_right_corner);
+    wCursor[WCUR_BOTTOMLEFTRESIZE] = XCreateFontCursor(dpy, XC_bottom_left_corner);
+    wCursor[WCUR_BOTTOMRIGHTRESIZE] = XCreateFontCursor(dpy, XC_bottom_right_corner);
+    wCursor[WCUR_VERTICALRESIZE] = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
+    wCursor[WCUR_HORIZONRESIZE] = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
     wCursor[WCUR_WAIT] = XCreateFontCursor(dpy, XC_watch);
     wCursor[WCUR_QUESTION] = XCreateFontCursor(dpy, XC_question_arrow);
     wCursor[WCUR_TEXT]     = XCreateFontCursor(dpy, XC_xterm); /* odd name???*/
+    wCursor[WCUR_SELECT] = XCreateFontCursor(dpy, XC_cross);
     
     /* emergency exit... */
     sig_action.sa_handler = handleSig;
@@ -853,22 +906,23 @@ StartUp(Bool defaultScreenOnly)
 	wMenuRestoreState(wScreen[j]);
 	
 	/* If we're not restarting restore session */
-	if (ws_state == NULL)
+	if (ws_state == NULL && !wPreferences.flags.norestore)
 	    wSessionRestoreState(wScreen[j]);
 	
-	
-	/* auto-launch apps */
-	if (!wPreferences.flags.nodock && wScreen[j]->dock) {
-	    wScreen[j]->last_dock = wScreen[j]->dock;
-	    wDockDoAutoLaunch(wScreen[j]->dock, 0);
-	}
-	/* auto-launch apps in clip */
-	if (!wPreferences.flags.noclip) {
-	    int i;
-	    for(i=0; i<wScreen[j]->workspace_count; i++) {
-		if (wScreen[j]->workspaces[i]->clip) {
-		    wScreen[j]->last_dock = wScreen[j]->workspaces[i]->clip;
-		    wDockDoAutoLaunch(wScreen[j]->workspaces[i]->clip, i);
+	if (!wPreferences.flags.noautolaunch) {
+	    /* auto-launch apps */
+	    if (!wPreferences.flags.nodock && wScreen[j]->dock) {
+		wScreen[j]->last_dock = wScreen[j]->dock;
+		wDockDoAutoLaunch(wScreen[j]->dock, 0);
+	    }
+	    /* auto-launch apps in clip */
+	    if (!wPreferences.flags.noclip) {
+		int i;
+		for(i=0; i<wScreen[j]->workspace_count; i++) {
+		    if (wScreen[j]->workspaces[i]->clip) {
+			wScreen[j]->last_dock = wScreen[j]->workspaces[i]->clip;
+			wDockDoAutoLaunch(wScreen[j]->workspaces[i]->clip, i);
+		    }
 		}
 	    }
 	}
