@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  *
@@ -26,9 +26,6 @@
 
 #include "wconfig.h"
 #include "GNUstep.h"
-#ifdef MWM_HINTS
-# include "motif.h"
-#endif
 #include "texture.h"
 #include "menu.h"
 #include "application.h"
@@ -48,7 +45,7 @@
 
 #define CLIENT_EVENTS (StructureNotifyMask | PropertyChangeMask\
 	| EnterWindowMask | LeaveWindowMask | ColormapChangeMask \
-	| FocusChangeMask)
+	| FocusChangeMask | VisibilityChangeMask)
 
 typedef enum {
     WFM_PASSIVE, WFM_NO_INPUT, WFM_LOCALLY_ACTIVE, WFM_GLOBALLY_ACTIVE
@@ -67,16 +64,37 @@ typedef enum {
  * that the value that will be used will be that of the leader foo.bar.
  * The attributes inspector must always save application wide options
  * in the name of the leader window, not the child.
+ * 
  */
 /*
  * All flags must have their default values = 0
+ * 
+ * New flag scheme:
+ * 
+ * user_flags, defined_flags
+ * client_flags
+ * 
+ * if defined window_flag then window_flag else client_flag
+ * 
  */
+
+#define WFLAGP(wwin, FLAG)	((wwin)->defined_user_flags.FLAG \
+					? (wwin)->user_flags.FLAG \
+					: (wwin)->client_flags.FLAG)
+
+#define WSETUFLAG(wwin, FLAG, VAL)	(wwin)->user_flags.FLAG = (VAL),\
+					(wwin)->defined_user_flags.FLAG = 1
+
 typedef struct {
     /* OpenStep */
     unsigned int no_titlebar:1;	       /* draw titlebar? */
     unsigned int no_resizable:1;
     unsigned int no_closable:1;
     unsigned int no_miniaturizable:1;
+    unsigned int no_border:1;	       /* 1 pixel border around window */
+#ifdef XKB_BUTTON_HINT
+    unsigned int no_language_button:1;
+#endif
 
     /* decorations */
     unsigned int no_resizebar:1;       /* draw the bottom handle? */
@@ -87,11 +105,12 @@ typedef struct {
 
     /* ours */
     unsigned int kill_close:1;	       /* can't send WM_DELETE_WINDOW */
-	
+
     unsigned int no_shadeable:1;
     unsigned int omnipresent:1;
     unsigned int skip_window_list:1;
-    unsigned int floating:1;	       /* put in NSFloatingWindowLevel */
+    unsigned int floating:1;	       /* put in WMFloatingLevel */
+    unsigned int sunken:1;	       /* put in WMSunkenLevel */
     unsigned int no_bind_keys:1;       /* intercept wm kbd binds
 					* while window is focused */
     unsigned int no_bind_mouse:1;      /* intercept mouse events
@@ -106,23 +125,42 @@ typedef struct {
     
     unsigned int always_user_icon:1;   /* ignore client IconPixmap or
 					* IconWindow */
-    
+
     unsigned int start_miniaturized:1;
     unsigned int start_hidden:1;
+    unsigned int start_maximized:1;
     unsigned int dont_save_session:1;  /* do not save app's state in session */
+
+    unsigned int full_maximize:1;
+
+    /*
+     * emulate_app_icon must be automatically disabled for apps that can
+     * generate their own appicons and for apps that have no_appicon=1
+     */
+    unsigned int emulate_appicon:1;
+
+    /* toolkit specific attribute flags. These are unchangeable and
+     * unconfigurable and must not be saved as state. */
+#ifdef OLWM_HINTS
+    unsigned int olwm_transient:1;
+    unsigned int olwm_warp_to_pin:1;
+#endif
 } WWindowAttributes;
 
 
+
 /*
  * Window manager protocols that both the client as we understand.
  */
 typedef struct {
     unsigned int TAKE_FOCUS:1;
-    unsigned int SAVE_YOURSELF:1;
     unsigned int DELETE_WINDOW:1;
-
+    unsigned int SAVE_YOURSELF:1;
     /* WindowMaker specific */
     unsigned int MINIATURIZE_WINDOW:1;
+#ifdef MONITOR_HEARTBEAT 
+    unsigned int HEARTBEAT:1;
+#endif
 } WProtocols;
 
 
@@ -135,7 +173,11 @@ typedef struct WWindow {
     struct WWindow *next;
 
     WScreen *screen_ptr;	       /* pointer to the screen structure */
-    WWindowAttributes window_flags;    /* window attribute flags */
+    WWindowAttributes user_flags;      /* window attribute flags set by user */
+    WWindowAttributes defined_user_flags;/* mask for user_flags */
+    WWindowAttributes client_flags;    /* window attribute flags set by app
+					* initialized with global defaults */
+
     struct InspectorPanel *inspector;  /*  pointer to attribute editor panel */
 
     struct WFrameWindow *frame;	       /* the frame window */
@@ -179,6 +221,15 @@ typedef struct WWindow {
     
     FocusMode focus_mode;	       /* type of keyboard input focus */
 
+#ifdef OLWM_HINTS_unfinished
+    struct WOLWindowState ol_window_state;
+#endif
+
+    long event_mask;		       /* the event mask thats selected */
+
+#ifdef MONITOR_HEARTBEAT
+    time_t last_beat;
+#endif
     struct {
 	/* state flags */
 	unsigned int mapped:1;	
@@ -187,6 +238,7 @@ typedef struct WWindow {
 	unsigned int hidden:1;
 	unsigned int shaded:1;
 	unsigned int maximized:2;
+	unsigned int omnipresent:1;
 
 	unsigned int semi_focused:1;
 	/* window type flags */
@@ -194,8 +246,11 @@ typedef struct WWindow {
 #ifdef SHAPE
 	unsigned int shaped:1;
 #endif
-	
+
 	/* info flags */
+	unsigned int is_gnustep:1;     /* 1 if the window belongs to a GNUstep
+					app */
+	
 	unsigned int buttons_dont_fit:1;
 	unsigned int rebuild_texture:1;/* the window was resized and 
 					* gradients should be re-rendered */
@@ -203,30 +258,64 @@ typedef struct WWindow {
 					   * window next time it's painted */
 	unsigned int icon_moved:1;     /* icon for this window was moved
 					* by the user */
-	unsigned int ignore_next_unmap:1;
 	unsigned int selected:1;       /* multiple window selection */
 	unsigned int skip_next_animation:1;
 	unsigned int internal_window:1;
 	unsigned int changing_workspace:1;
 
         unsigned int inspector_open:1; /* attrib inspector is already open */
-	
+
 	unsigned int destroyed:1;      /* window was already destroyed */
+
+	unsigned int menu_open_for_me:1;   /* window commands menu */
+
+	unsigned int waiting_save_ack:1;   /* waiting for SAVE_YOURSELF ack */
+
+	unsigned int obscured:1;       /* window is obscured */
+
+	unsigned int dragged_while_fmaximized;
+
+	unsigned int user_changed_width:1;
+	unsigned int user_changed_height:1;
+	unsigned int wm_name_changed:1;
+
+#ifdef KWM_HINTS
+	unsigned int kwm_hidden_for_modules:1;
+	unsigned int kwm_managed:1;
+	unsigned int kwm_menubar:1;
+#endif
+#ifdef OLWM_HINTS
+	unsigned int olwm_push_pin_out:1;/* emulate pushpin behaviour */
+	unsigned int olwm_limit_menu:1;
+#endif
     } flags;		/* state of the window */
 
-    char *icon_text;
     struct WIcon *icon;		       /* icon info for the window */
     int icon_x, icon_y;		       /* position of the icon */
 } WWindow;
 
 
+#define IS_OMNIPRESENT(w) ((w)->flags.omnipresent ^ WFLAGP(w, omnipresent))
+
+#define WINDOW_LEVEL(w) ((w)->frame->core->stacking->window_level)
+
+
+
+/*
+ * Changes to this must update wWindowSaveState/getSavedState
+ * 
+ */
 typedef struct WSavedState {
     int workspace;
     int miniaturized;
     int shaded;
     int hidden;
     int use_geometry;
-    int x, y, w, h;
+    int x;
+    int y;
+    unsigned int w;
+    unsigned int h;
+    unsigned window_shortcuts; /* mask like 1<<shortcut_number */
 } WSavedState;
 
 
@@ -240,18 +329,23 @@ typedef struct WWindowState {
 } WWindowState;
 
 
+typedef void* WMagicNumber;
+
+
+
 void wWindowDestroy(WWindow *wwin);
 WWindow *wWindowCreate();
 
 #ifdef SHAPE
 void wWindowSetShape(WWindow *wwin);
+void wWindowClearShape(WWindow *wwin);
 #endif
 
 WWindow *wManageWindow(WScreen *scr, Window window);
 
-void wUnmanageWindow(WWindow *wwin, int restore);
+void wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed);
 
-void wWindowFocus(WWindow *wwin);
+void wWindowFocus(WWindow *wwin, WWindow *owin);
 void wWindowUnfocus(WWindow *wwin);
 void wWindowConstrainSize(WWindow *wwin, int *nwidth, int *nheight);
 void wWindowConfigure(WWindow *wwin, int req_x, int req_y, 
@@ -280,5 +374,25 @@ WWindow *wManageInternalWindow(WScreen *scr, Window window, Window owner,
 			       char *title, int x, int y, 
 			       int width, int height);
 
+void wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace);
+
+void wWindowUpdateGNUstepAttr(WWindow *wwin, GNUstepWMAttributes *attr);
+
+void wWindowMap(WWindow *wwin);
+
+void wWindowUnmap(WWindow *wwin);
+
+Bool wWindowCanReceiveFocus(WWindow *wwin);
+
+void wWindowDeleteSavedStatesForPID(pid_t pid);
+
+WMagicNumber wWindowAddSavedState(char *instance, char *class, char *command,
+                                  pid_t pid, WSavedState *state);
+
+WMagicNumber wWindowGetSavedState(Window win);
+
+void wWindowDeleteSavedState(WMagicNumber id);
+
+Bool wWindowObscuresWindow(WWindow *wwin, WWindow *obscured);
 
 #endif
